﻿// Nova.CodeDOM usage examples.
// Copyright (C) 2007-2011 Inevitable Software, all rights reserved.

using System;
using System.Diagnostics;
using System.Reflection;

using Nova.Analysis;  // Required for Metrics
using Nova.CodeDOM;   // The CodeDOM namespace

namespace Nova.Examples
{
    /// <summary>
    /// This program contains examples for using the Nova.CodeDOM C# object model library.
    /// </summary>
    class Program
    {
        #region /* MAIN METHOD */

        static void Main()
        {
            // Determine the application name and if we're running in VS
            string appName = AppDomain.CurrentDomain.FriendlyName;
            if (appName.EndsWith(".exe"))
                appName = appName.Substring(0, appName.Length - 4);
            if (appName.EndsWith(".vshost"))
                appName = appName.Substring(0, appName.Length - 7);

            Version appVersion = typeof(Program).Assembly.GetName().Version;
            string appDescription = appName + " " + appVersion.Major + "." + appVersion.Minor;
            Console.WriteLine(appDescription + "  -  C# CodeDOM Examples.\nCopyright (C) 2011 Inevitable Software, all rights reserved.");

            // Execute all public examples in this class
            Console.WriteLine("\nExecuting Nova.CodeDOM examples...\n");
            MethodInfo[] methodInfos = Assembly.GetExecutingAssembly().GetType("Nova.Examples.Program").GetMethods(BindingFlags.Public | BindingFlags.Static);
            foreach (var methodInfo in methodInfos)
            {
                Log.WriteLine("===== Executing Example: " + methodInfo.Name + " =====");
                try
                {
                    methodInfo.Invoke(null, null);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("EXCEPTION: " + ex.Message);
                }
                Log.WriteLine("===== " + methodInfo.Name + " Completed =====\n");
            }

            // Wait for a key press before exiting if running in VS
            if (Debugger.IsAttached)
            {
                Console.WriteLine("DONE.  Press any key to exit.");
                Console.ReadKey();
            }
        }

        #endregion

        #region /* LOGGING & CONFIGURATION NOTES */

        // Nova.CodeDOM includes a logging class that logs status messages to the Console Window by default.
        // Use the Log class to log your own messages like this:
        //     Log.WriteLine("message");
        // To intercept these log messages, you may install your own handler method of type 'void (string, string)':
        //     Log.SetLogWriteLineCallback(LoggingMethod);

        // Various runtime options are available in the provided ".config" file, and are documented there.
        // These options may also be controlled manually at runtime by setting the static member, such as:
        //     Log.DetailLogging = true;

        #endregion

        #region /* LOADING EXAMPLES */

        /// <summary>
        /// Load an entire Solution, parsing and resolving it.
        /// </summary>
        public static void LoadSolution()
        {
            Solution solution = Solution.Load("..\\..\\Nova.Examples.sln");
            if (solution != null)
                Log.WriteLine("Solution '" + solution.Name + "' successfully loaded, parsed, resolved.");

            // If successful, a Solution object is returned, otherwise null.
            // By default, a Complete load is done, but this can be overridden.
            // A 'Complete' load of a Solution goes through the following steps:
            //   - Parse the Solution file, creating a Solution object and a list of ProjectEntry objects.
            //   - Parse each Project file, creating a Project object and loading the lists of References and code files (CodeUnits).
            //   - Resolve all References for each project, and determine the project dependencies.
            //   - Load all referenced assemblies for each project (in dependency order).
            //   - Load all types from all referenced assemblies for each project (in dependency order).
            //   - Parse all code files for each project (in dependency order), first loading all types from any referenced projects.
            //   - Resolve all symbolic references in all code files for each project (in dependency order).
            // The 'LoadOnly' option stops this process after parsing the Solution and Project files and resolving References, which
            // is useful if only the solution and/or project files are being viewed, analyzed, or edited.  The 'DoNotResolve' option
            // skips resolving symbolic references, which is useful if that step is not required, such as when only formatting code
            // or calculating metrics which don't rely on resolved references.

            // Source code is loaded into a semantic tree of pure code objects, without separate objects for syntax (such as braces),
            // but formatting information and comments are retained on these objects.  By default, code will be re-formatted according
            // to the default formatting rules when displayed or saved, but this can be controlled with options.  Also, newlines are
            // generally retained as-is in loaded code, so that changes will be minimal unless re-formatting is explicitly performed.
        }

        /// <summary>
        /// Load a Solution WITHOUT parsing or resolving it (useful to only analyze and/or modify the Solution and/or Projects).
        /// </summary>
        public static void LoadSolutionLoadOnly()
        {
            Solution solution = Solution.Load("..\\..\\Nova.Examples.sln", LoadOptions.LoadOnly);
            if (solution != null)
                Log.WriteLine("Solution '" + solution.Name + "' successfully loaded without parsing or resolving source files.");
        }

        /// <summary>
        /// Load a Solution and parse it, but don't resolve it (useful if only formatting or other processes that don't require resolving symbolic references).
        /// </summary>
        public static void LoadSolutionDoNotResolve()
        {
            Solution solution = Solution.Load("..\\..\\Nova.Examples.sln", LoadOptions.DoNotResolve);
            if (solution != null)
                Log.WriteLine("Solution '" + solution.Name + "' successfully loaded and parsed, but not resolved.");
        }

        /// <summary>
        /// Load a single Project, parsing and resolving it (useful for working on a single project of a solution).
        /// Any missing project references will be treated as assembly references so that symbols can be resolved.
        /// </summary>
        public static void LoadProject()
        {
            Project project = Project.Load("..\\..\\Nova.Examples.csproj");
            if (project != null)
                Log.WriteLine("Project '" + project.Name + "' successfully loaded, parsed, resolved.");

            // If successful, a Project object is returned, otherwise null.
            // By default, a Complete load is done, but this can be overridden.
            // Loading a Project directly goes through the following steps:
            //   - Parse the Project file, creating a Project object and loading the lists of References and code files (CodeUnits).
            //   - Resolve all References for the project.
            //   - Load all referenced assemblies.
            //   - Load all types from all referenced assemblies.
            //   - Parse all code files.
            //   - Resolve all symbolic references in all code files.
            // The 'LoadOnly' option stops this process after parsing the Project file and resolving References, which is useful
            // if only the project file itself is being viewed, analyzed, or edited.  The 'DoNotResolve' option skips resolving
            // symbolic references, which is useful if that step is not required, such as when only formatting code or calculating
            // metrics which don't rely on resolved references.
        }

        /// <summary>
        /// Load a single CodeUnit (source file), parsing and resolving it.
        /// </summary>
        public static void LoadCodeUnit()
        {
            // When loading a CodeUnit directly, all external symbolic references will fail to resolve, because we have no Project
            // and therefore no assembly references.  Internal references (to locals, parameters, members of the same type, etc)
            // will still be resolved.  Turn off logging of messages so all the resulting error messages aren't logged.
            CodeUnit codeUnit = CodeUnit.Load("..\\..\\Program.cs", LoadOptions.Complete & ~LoadOptions.LogMessages);
            if (codeUnit != null)
                Log.WriteLine("File '" + codeUnit.Name + "' successfully loaded, parsed, resolved.");
        }

        /// <summary>
        /// Load a code fragment, parsing and resolving it.
        /// </summary>
        public static void LoadCodeFragment()
        {
            // A code fragment can be a type or method declaration, or just a chunk of code.  Formatting is not that
            // important, including newlines (although newlines - or the lack of them - will be preserved unless the
            // code is specifically re-formatted).  Invalid code will produce parse errors.  
            const string fragment = "int sum = 0; for (int i = 1; i < 10; ++i) sum += i;";

            // As in the LoadCodeUnit example, external references will not be resolved since there are no assembly references
            CodeUnit codeUnit = CodeUnit.LoadFragment(fragment, "Fragment");
            if (codeUnit != null)
                Log.WriteLine("Code fragment '" + codeUnit.Name + "' successfully parsed, resolved.");
        }

        /// <summary>
        /// Load multiple CodeUnits with resolving of external references.
        /// </summary>
        public static void LoadCodeUnits()
        {
            // To load one or more CodeUnits with external references resolved, create a 'dummy' parent Project for the CodeUnits,
            // and add any desired assembly references to it.
            Project project = new Project("Miscellaneous Files", null);
            CodeUnit codeUnit1 = project.AddFile("..\\..\\Program.cs");  // Add files like this
            const string fragment = "using System; Console.WriteLine(\"TEST\");";
            CodeUnit codeUnit2 = project.CreateCodeUnit(fragment, "Fragment");  // Add code fragments like this

            // We want external references resolved in this case, so add assembly references and load their types
            project.AddDefaultAssemblyReferences();  // Add commonly used assembly references
            project.AddAssemblyReference("Nova.CodeDOM");  // Add additional assembly references like this
            project.LoadReferencedAssembliesAndTypes();

            // Parse and resolve all code units in the project
            project.ParseAndResolveCodeUnits();

            // In this example, there is no Solution, so we can't benefit from it's single CodeAnnotations collection
            // of all messages for all source files.  We could add a dummy Solution to the Project to get this, or look
            // at the ListedAnnotations collection on each CodeUnit.  In this case, we'll just call a helper method on
            // each CodeUnit that displays message counts along with any errors or warnings.
            codeUnit1.LogMessageCounts(true);
            codeUnit2.LogMessageCounts(true);
        }

        #endregion

        #region /* MANUAL CODE CREATION & SAVING EXAMPLES */

        /// <summary>
        /// Manually create a new Solution, Project, and CodeUnit (with code in it) and save them.
        /// </summary>
        public static void CreateNewSolution()
        {
            // Create a new solution and project
            const string path = "..\\..\\Generated\\";
            Solution solution = new Solution(path + "Generated");  // file extension will default to '.sln' if omitted
            Project project = solution.CreateProject(path + "Generated");  // file extension will default to '.csproj' if omitted
            project.OutputType = Project.OutputTypes.Exe;  // Output type will default to Library if not set

            // In order to have external symbolic references resolved, assembly references must be added, then loaded.
            // However, this is not necessary in this case, since we are only saving the generated code, and UnresolvedRefs
            // will save just fine using their text names.
            //project.AddDefaultAssemblyReferences();  // Add commonly used assembly references
            //project.LoadReferencedAssembliesAndTypes();

            // Add a file to the project, and put some code in it
            CodeUnit codeUnit = project.CreateCodeUnit(path + "Program");  // file extension will default to '.cs' if omitted
            codeUnit.Add(
                new UsingDirective(project.ParseName("System")),
                new UsingDirective(project.ParseName("System.Collections.Generic")),
                new UsingDirective(project.ParseName("System.Linq")),
                new UsingDirective(project.ParseName("System.Text")),
                new NamespaceDecl(project.ParseName("Generated"))
                    {
                        Body = { new ClassDecl("Program")
                                     {
                                         Body = { new MethodDecl("Main", typeof(void), Modifiers.Static, new ParameterDecl("args", typeof(string[])))
                                                      {
                                                          Body = { new Comment("Add code here") }
                                                      }}
                                     }}
                    });

            // External references will be unresolved, unless we add and load assembly references as shown up above
            Log.WriteLine("UnresolvedRefs = " + codeUnit.CalculateMetrics().UnresolvedReferences);

            // If code is simple enough, object initializers can be used as shown above, but when there are symbolic references,
            // objects need to be assigned to local variables so that references can be easily generated.  This is shown in the
            // next example, which generates a code fragment.  Also, note the use of 'project.ParseName()' in the UsingDirectives
            // (which aren't necessary in this case, just shown as examples) - this is a helper method that parses a string into
            // an expression of NamespaceRef and/or TypeRefs and Dot operators if present.  If the namespaces or types can't be
            // found in scope (in the referenced assemblies), then UnresolvedRefs will be used.  Manually created code is usually
            // created with resolved references, and so needs no resolving, but UnresolvedRefs can also be used if desired.

            // Manually created objects will be default formatted - there's no reason to worry about braces, parenthesis, commas,
            // semi-colons, newlines, whitespace, or anything else that is purely syntax-related and not semantic.  However, the
            // default formatting can be overridden using various properties, such as NewLines, HasBraces, HasParens.  You may
            // use '.IsFirstOnLine = true' to force an object to start on a new line (or '.NewLines = 1'), or '.IsSingleLine = true'
            // to force an Expression or Statement to be formatted on a single line.

            // For more examples of manual code creation, see ManualTests.GenerateFullTest() in the Nova.Test solution, which
            // generates a replica of the related FullTest.cs file, which includes most C# language features.

            // Save the entire solution, including all projects and files
            solution.SaveAll();

            // You may also save a single project and all of its files using 'project.SaveAll()', or you may save just an individual
            // solution, project, or file by calling '.Save()' on it.  Also, '.SaveAs(string)' methods are provided to save under
            // a different file name.
        }

        /// <summary>
        /// Manually create a code fragment, and convert it to a string.
        /// </summary>
        public static void CreateCodeFragment()
        {
            // Any code object can be created without a parent, and with whatever children are desired.  For example, you might
            // create a ClassDecl or a MethodDecl, or you can create a BlockDecl (a block of statements delimited by braces, as
            // shown below), or you can create an individual Statement or Expression of any kind.
            var sum = new LocalDecl("sum", typeof(int), 0);
            var flag = new LocalDecl("flag", typeof(bool), false) { EOLComment = "some flag" };
            var i = new LocalDecl("i", typeof(int), 1);
            var @for = new For(i, new LessThan(i, 10), new Increment(i),
                new Block(new AddAssign(sum, i), new Assignment(flag, true))) { Comment = "calculate a sum" };
            var block = new BlockDecl(sum, flag, @for);

            // When a named code object (such as 'i' above), is used where an Expression is expected, there are implicit conversion
            // operators which create SymbolicRefs (in this case, a LocalRef), avoiding the need to call '.CreateRef()' each time.
            // Likewise, Types (such as 'typeof(int)') used for Expressions are implicitly converted to TypeRefs (avoiding the need to
            // use 'new TypeRef(typeof(int))', and literals (such as '0' or 'false') are implicitly converted to Literals (avoiding
            // the need to use 'new Literal(0)' or 'new Literal(false)'.

            // Only Solution, Project, and CodeUnit objects implement the IFile interface and are mapped to files.  For all other code
            // objects, there is no '.Save()' method.  However, you may convert any code object to a string using 'AsString()'
            // (the 'ToString()' method generates only a 'description' instead of the full object, because otherwise it would cause
            // performance issues when working in the debugger if all objects rendered all of their child objects).
            Log.WriteLine(block.AsString());
        }

        #endregion

        #region /* SEARCHING EXAMPLES */

        /// <summary>
        /// Finding code objects.
        /// </summary>
        public static void FindCodeObjects()
        {
            // Various 'Find' methods are provided on various CodeDOM types to find immediate child code objects at a particular
            // point in a code tree, examples of which are shown below.  Global finds or searches are addressed in later examples.

            Solution solution = Solution.Load("..\\..\\Nova.Examples.sln");
            if (solution != null)
            {
                // Find a project in a solution by name
                Project project = solution.FindProject("Nova.Examples");
                if (project != null)
                {
                    Log.WriteLine("Found " + project);

                    // Find a code unit in a project by name
                    CodeUnit codeUnit = project.FindCodeUnit("Program.cs");
                    if (codeUnit != null)
                        Log.WriteLine("Found " + codeUnit);

                    // Find a namespace in a project by full name
                    Namespace @namespace = project.FindNamespace("Nova.Examples");
                    if (codeUnit != null)
                        Log.WriteLine("Found " + @namespace);

                    // Find a TypeDecl in a project
                    var typeDecl1 = project.Find("Nova.Examples.Program") as TypeDecl;
                    if (typeDecl1 != null)
                        Log.WriteLine("Found TypeDecl: " + typeDecl1.Name);

                    // Find a Type in a project
                    var type = project.Find("System.Console") as Type;
                    if (type != null)
                        Log.WriteLine("Found Type: " + type.Name);

                    // Find a TypeDecl in a namespace
                    var typeDecl = @namespace.Find("Program") as TypeDecl;
                    if (typeDecl != null)
                    {
                        Log.WriteLine("Found " + typeDecl);

                        // Find a MethodDecl in a TypeDecl by name
                        var methodDecl = typeDecl.Find("FindCodeObjects") as MethodDecl;
                        if (methodDecl != null)
                        {
                            Log.WriteLine("Found " + methodDecl);

                            // Find a LocalDecl in a MethodDecl by name
                            var localDecl = methodDecl.Find("solution") as LocalDecl;
                            if (localDecl != null)
                            {
                                Log.WriteLine("Found " + localDecl);

                                // Find the Parent MethodDecl of a code object
                                var parentMethod = localDecl.FindParentMethod();
                                if (parentMethod != null)
                                    Log.WriteLine("Found parent " + parentMethod);

                                // Find the Parent NamespaceDecl of a code object
                                var parentNamespaceDecl = localDecl.FindParent<NamespaceDecl>();
                                if (parentNamespaceDecl != null)
                                    Log.WriteLine("Found parent " + parentNamespaceDecl);
                            }

                            // Find the first 'if' statement in a MethodDecl
                            var @if = methodDecl.Find<If>();
                            if (@if != null)
                            {
                                Log.WriteLine("Found " + @if);

                                // Find all child 'if' statements in the body of a parent 'if' statement
                                foreach (var statement in @if.Body)
                                {
                                    if (statement is If)
                                        Log.WriteLine("Found " + statement);
                                }
                            }
                        }
                    }
                }
            }

            // Helper methods are provided on various SymbolicRef types to find types when manually generating code
            // which conveniently return an UnresolvedRef instead of null if the item isn't found.  They can also
            // be passed an UnresolvedRef as input, so the output of one call can be passed as the input of another
            // without any checks for errors.  For examples, see ManualTests.GenerateFullTest().
            // These methods include the following signature patterns:
            //
            // Get a reference to a type or namespace in the current project instance:
            //     SymbolicRef Project.FindRef(string name)
            //
            // Find a type in a namespace or parent type, where NorT is a Namespace or Type/TypeDecl, or an Alias to one
            //     SymbolicRef TypeRef.Find(NorT type, string name)
            //
            // Find a member of a type, where TYPE is a Type, TypeDecl, TypeRefBase, or type Alias:
            //     SymbolicRef MethodRef.Find(TYPE type, string name)
            //     SymbolicRef ConstructorRef.Find(TYPE type, params TypeRefBase[] parameterTypes)
            //     SymbolicRef PropertyRef.Find(TYPE type, string name)
            //     SymbolicRef FieldRef.Find(TYPE type, string name)
            //
            // Find a parameter of a method, where METHOD is a MethodInfo or MethodDeclBase
            //     SymbolicRef ParameterRef.Find(METHOD method, string name)
        }

#if NEXT_RELEASE
        /// <summary>
        /// Find all references to code object (usages).
        /// </summary>
        public static void FindReferences()
        {
            // Find all references (usages) of the specified named code object
            //List<SymbolicRef> CodeObject.FindReferences(INamedCodeObject)
        }

        /// <summary>
        /// Find all code objects by name and/or type.
        /// </summary>
        public static void FindAllCodeObjects()
        {
            // Find all INamedCodeObjects with the specified name
            //List<CodeObject> CodeObject.FindAll(string)

            // Find all CodeObjects of a particular type
            //List<CodeObject> CodeObject.FindAll<T>()

            // Find all CodeObjects of a particular type with the specified name
            //List<CodeObject> CodeObject.FindAll<T>(string)
        }

        /// <summary>
        /// Search for text strings and/or patterns.
        /// </summary>
        public static void TextSearch()
        {
            // Text search within names, string literals, comments
            // Scope: Solution, Project, CodeUnit, fragment
            // Options: Match whole word, Case insensitive, regular expressions
        }
#endif

        #endregion

        #region /* CODE MODIFICATION EXAMPLES */

        /// <summary>
        /// Modify code objects.
        /// </summary>
        public static void ModifyCodeObjects()
        {
            // Renaming a named code object is trivial - just assign a new name, and all resolved references will
            // automatically make use of the new name (for name changes, it's obviously very important that all
            // symbolic references are resolved first).  Code objects have various other properties which can also
            // be directly assigned.  Some code objects have direct children code objects (such as the various parts
            // of a For loop) which can be directly assigned to LocalDecls or Expressions.  Many (such as all
            // BlockStatements) have a Body which is a collection of child code objects which can be manipulated
            // using methods such as Add(), Remove(), and Insert().

            // Here are some examples using a small code fragment:
            const string fragment = @"
/// <see cref=""OldClass""/>
private class OldClass
{
    public OldClass(int arg)
    { }

    private void ToBeDeletedMethod()
    { }

    /// <param name=""oldArg"">The argument.</param>
    private void Method(int oldArg = -1)
    {
        OldClass x = new OldClass(oldArg);
        for (int i = oldArg; i < 10; ++i)
            x.Method(oldArg);
    }
}
";
            CodeUnit codeUnit = CodeUnit.LoadFragment(fragment, "Fragment");
            if (codeUnit != null)
            {
                var classDecl = codeUnit.Find("OldClass") as ClassDecl;
                if (classDecl != null)
                {
                    // Rename 'OldClass' to 'NewClass' and change it to 'public'
                    classDecl.Name = "NewClass";
                    classDecl.IsPublic = true;

                    var methodDecl = classDecl.Find("Method") as MethodDecl;
                    if (methodDecl != null)
                    {
                        // Change 'Method' to 'public', and change its return type from 'void' to 'int'
                        methodDecl.IsPublic = true;
                        methodDecl.ReturnType = typeof(int);

                        // Rename the parameter of 'Method' from 'oldArg' to 'newArg', change its type from 'int' to 'long',
                        // and change its initialization from '-1' to '0'.
                        var parameterDecl = methodDecl.Parameters[0];
                        parameterDecl.Name = "newArg";
                        parameterDecl.Type = typeof(long);
                        parameterDecl.Initialization = 0;
                    }

                    // Insert a new method after the constructor
                    classDecl.Insert(1, new MethodDecl("InsertedMethod", typeof(void)));

                    // Remove a method
                    var toBeDeleted = classDecl.Find("ToBeDeletedMethod") as CodeObject;
                    if (toBeDeleted != null)
                        classDecl.Remove(toBeDeleted);
                }

                // Insert an enum into the CodeUnit before the class
                codeUnit.Insert(0, new EnumDecl("Enum") { MemberDecls = { new EnumMemberDecl("One"), new EnumMemberDecl("Two") } });

                // Display the modified code
                Log.WriteLine(codeUnit.AsString());
            }
        }

        #endregion

        #region /* METRICS EXAMPLES */

        /// <summary>
        /// Generating metrics.
        /// </summary>
        public static void GenerateMetrics()
        {
            Solution solution = Solution.Load("..\\..\\Nova.Examples.sln");
            if (solution != null)
            {
                // Basic metrics can be calculated for any individual code object by calling ".CalculateMetrics()" on it.

                // Here's an example of some metrics for an entire Solution:
                Metrics metrics = solution.CalculateMetrics();
                Log.WriteLine(string.Format("Solution '{0}': {1:N0} projects; {2:N0} files; {3:N0} lines; {4:N0} SLOC; {5:N0} types; {6:N0} code objects; {7:N2} code objects per SLOC",
                    solution.Name, metrics.Projects, metrics.Files, metrics.Lines, metrics.SLOC, metrics.Types, metrics.CodeObjects, metrics.CodeObjectsPerSLOC));

                // Here's an example of some metrics for an individual source file:
                Project project = solution.FindProject("Nova.Examples");
                CodeUnit codeUnit = project.FindCodeUnit("Program.cs");
                metrics = codeUnit.CalculateMetrics();
                Log.WriteLine(string.Format("CodeUnit '{0}': {1:N0} lines; {2:N0} SLOC; {3:N0} types; {4:N0} code objects; {5:N2} code objects per SLOC",
                    codeUnit.Name, metrics.Lines, metrics.SLOC, metrics.Types, metrics.CodeObjects, metrics.CodeObjectsPerSLOC));
            }
        }

        #endregion
    }
}
