﻿// The Nova Project by Ken Beckett.
// Copyright (C) 2007-2011 Inevitable Software, all rights reserved.

// This file contains test code that manually generates objects using the Nova.CodeDOM library.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Threading;

using Nova.CodeDOM;
using Nova.Utilities;
using Attribute = Nova.CodeDOM.Attribute;
using Label = Nova.CodeDOM.Label;
using LambdaExpression = Nova.CodeDOM.LambdaExpression;
using ParameterModifier = Nova.CodeDOM.ParameterModifier;
using Switch = Nova.CodeDOM.Switch;

namespace Nova.Test
{
    /// <summary>
    /// Manual CodeObject tests.
    /// </summary>
    public class ManualTests
    {
        public static CodeObject GenerateSimpleTest(string testName, Project project)
        {
            // int x;
            // x = 1;
            // int y = x + 1;
            // x = y * 2;
            // const string t = "test";
            LocalDecl x, y;
            // Use a BlockDecl as a parent for multiple top-level statements
            CodeObject testCode = new BlockDecl(
                x = new LocalDecl("x", typeof(int)),
                new Assignment(x, 1),
                y = new LocalDecl("y", typeof(int), new Add(x, 1)),
                new Assignment(x, new Multiply(y, 2)),
                new LocalDecl("t", typeof(string), Modifiers.Const, "test")
                );
            return testCode;
        }

        public static CodeObject GenerateMethodTest(string testName, Project project)
        {
            // string CalculateGrade(string name, int score)
            // {
            //     string result = name;
            //     if (score < 70)
            //     {
            //         result += " FAILED";
            //         bool passed = false;
            //     }
            //     else
            //         result += " PASSED";
            //     return result;
            // }
            var name = new ParameterDecl("name", typeof(string));
            var score = new ParameterDecl("score", typeof(int));
            var method = new MethodDecl("CalculateGrade", typeof(string), name, score);
            var result = new LocalDecl("result", typeof(string), name);
            method.Add(result,
                new If(new LessThan(score, 70), new Block(new AddAssign(result, " FAILED"), new LocalDecl("passed", typeof(bool), false)),
                    new Else(new AddAssign(result, " PASSED"))), new Return(result));
            return method;
        }

        /// <summary>
        /// Manually generate code objects that replicate the code in FullTest.cs.
        /// </summary>
        public static CodeUnit GenerateFullTest(string testName, Project project)
        {
            var codeUnit = project.CreateCodeUnit(testName);

            codeUnit.Add(new Comment("The Nova Project by Ken Beckett.\nCopyright (C) 2007-2011 Inevitable Software, all rights reserved."),
                new Comment("This file contains test code for the Nova.CodeDOM library - it's not designed to be\nexecuted, only parsed and resolved."),
                new PragmaWarningDirective(PragmaWarningAction.Disable),
                new UnDefSymbol("JUST_TESTING") { EOLComment = "undefine symbol" },
                new DefineSymbol("UNDOCUMENTED") { EOLComment = "define symbol" });

            var rootNamespaceEnvV1 = project.FindReferenceAliasNamespace("EnvV1");
            var envV1 = new ExternAlias(rootNamespaceEnvV1) { NewLines = 1 };
            var namespace_EnvDTE80 = rootNamespaceEnvV1.FindNamespace("EnvDTE80");
            var type_vsBuildErrorLevel = (Type)namespace_EnvDTE80.Find("vsBuildErrorLevel");
            codeUnit.Add(new Comment("Test extern aliases"), envV1, new ExternAlias(project.FindReferenceAliasNamespace("EnvV2")),
                new Alias("BuildErrorLevel", Lookup.Create(envV1, namespace_EnvDTE80, type_vsBuildErrorLevel)) { NewLines = 1 },  // build-up expression manually
                new Alias("HTMLViews", project.ParseName("EnvV2::EnvDTE90.vsHTMLViews")));  // parse expression from string

            codeUnit.Add(
                new UsingDirective(project.ParseName("System")),
                new UsingDirective(project.ParseName("System.Collections")),
                new UsingDirective(project.ParseName("System.Collections.Generic")),
                new UsingDirective(project.ParseName("System.ComponentModel")),
                new UsingDirective(project.ParseName("System.Diagnostics")),
                new UsingDirective(project.ParseName("System.Linq")),
                new UsingDirective(project.ParseName("System.Linq.Expressions")),
                new UsingDirective(project.ParseName("System.Reflection")),
                new UsingDirective(project.ParseName("Nova.CodeDOM"))
                );
            var win_alias = new Alias("Win", project.ParseName("System.Windows")) { EOLComment = "namespace alias" }; codeUnit.Add(win_alias);
            var app_alias = new Alias("App", project.ParseName("System.Windows.Application")) { EOLComment = "type alias" }; codeUnit.Add(app_alias);
            var attribute_alias = new Alias("Attribute", project.ParseName("System.Attribute")); codeUnit.Add(attribute_alias);

            codeUnit.Add(new Attribute(AttributeTarget.Assembly, new Call(new Dot(project.ParseName("System.Reflection"),
                ConstructorRef.Find(typeof(AssemblyInformationalVersionAttribute), typeof(string))), "0.9")) { EOLComment = "global attribute" });


            // namespace Test.Full
            var @namespace = new NamespaceDecl(project.ParseName("Test.Full"));
            codeUnit.Add(@namespace);
            @namespace.Add(new UsingDirective(project.ParseName("System.Runtime.InteropServices")));


            // class CustomAttribute
            var classCustomAttribute = new ClassDecl("CustomAttribute", Modifiers.None, typeof(Attribute)) { DocComment = new DocSummary("\nThis is a custom attribute.\n") };
            var customAttribute_Message = new PropertyDecl("Message", typeof(string), Modifiers.Public) { IsAutomatic = true };
            var customAttribute_Value = new PropertyDecl("Value", typeof(int), Modifiers.Public) { IsAutomatic = true };
            classCustomAttribute.Add(customAttribute_Message, customAttribute_Value);
            classCustomAttribute.AttachAnnotation(new Attribute(ConstructorRef.Find(typeof(AttributeUsageAttribute), typeof(AttributeTargets)),
                new Dot(typeof(AttributeTargets), EnumMemberRef.Find(typeof(AttributeTargets), "All"))));
            @namespace.Add(classCustomAttribute);


            // public enum TestEnum
            var TestEnum = new EnumDecl("TestEnum", Modifiers.Public, typeof(byte)) { Comment = "This is a top-level enum declaration" };
            @namespace.Add(TestEnum);
            //testEnum.AttachAnnotation(new Attribute(typeof(FlagsAttribute)));
            TestEnum.IsBitFlags = true;  // This is the easy way to add the Flags attribute
            TestEnum.Add("Zero");
            var testEnum_One = new EnumMemberDecl("One", 1) { EOLComment = "EOL comment" };
            testEnum_One.AttachAnnotation(new IfDirective(true));
            testEnum_One.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsPostfix);
            var testEnum_Two = new EnumMemberDecl("Two") { EOLComment = "EOL comment" };
            testEnum_Two.AttachAnnotation(new IfDirective(true));
            testEnum_Two.AttachComment("Regular comment");
            testEnum_Two.AttachAnnotation(new Attribute(ConstructorRef.Find(typeof(DescriptionAttribute), typeof(string)), "two"));
            testEnum_Two.AttachAnnotation(new ElseDirective { SkippedText = "Too," }, AnnotationFlags.IsPostfix);
            testEnum_Two.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsPostfix);
            var testEnum_Four = new EnumMemberDecl("Four", 4);
            TestEnum.Add(testEnum_One, testEnum_Two, testEnum_Four);


            // public class TestClass : ArrayList, IDisposable
            var TestClass = new ClassDecl("TestClass", Modifiers.Public) { DocComment = new DocSummary("\nThis is a class.\n") };
            TestClass.AddBaseTypes(new TypeRef(typeof(ArrayList), true) { Comment = "Comment on base-type list" }, typeof(IDisposable));
            TestClass.AttachAnnotation(new Attribute(new Call(ConstructorRef.Find(classCustomAttribute), new Assignment(customAttribute_Message, "test"),
                new Assignment(customAttribute_Value, 3))) { EOLComment = "class attribute" });
            TestClass.AttachAnnotation(new IfDirective(true));
            TestClass.AttachAnnotation(new ElseDirective { SkippedText = "class TestClass" }, AnnotationFlags.IsInfix1);
            TestClass.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsInfix1);
            @namespace.Add(TestClass);


            // ENUMS

            TestClass.Add(new Comment("ENUMS"));

            // enum E1
            var enumE1 = new EnumDecl("E1") { Comment = "This is a nested enum declaration" };
            enumE1.Add("One", 1);
            var enumE1_two = new EnumMemberDecl("Two", 2);
            enumE1.Add(enumE1_two);
            TestClass.Add(enumE1);

            // enum SingleEnum
            var enumSingleEnum = new EnumDecl("SingleEnum");
            enumSingleEnum.Add("LonelyValue");
            TestClass.Add(enumSingleEnum);


            // CONSTANTS & FIELDS

            TestClass.Add(new Comment("CONSTANTS & FIELDS"));
            var _maxItems = new FieldDecl("MaxItems", typeof(int), Modifiers.Public | Modifiers.Const, 100);
            TestClass.Add(_maxItems);
            var _test = new FieldDecl("_test", typeof(string), Modifiers.Private) { NewLines = 1 };
            _test.AttachAnnotation(new Attribute(ConstructorRef.Find(typeof(ObsoleteAttribute), typeof(string)), "don't use") { EOLComment = "field attribute" });
            TestClass.Add(_test);
            FieldDecl _instance = new FieldDecl("_instance", TestClass, Modifiers.Private | Modifiers.Static); TestClass.Add(_instance);
            FieldDecl _2darray = new FieldDecl("_2darray", typeof(double[,]), Modifiers.Private); TestClass.Add(_2darray);
            var mfd = new MultiFieldDecl(typeof(int), Modifiers.Private, "_a", "_b"); TestClass.Add(mfd);
            var _c = new FieldDecl("_c", typeof(int), Modifiers.Private, 0);
            var _d = new FieldDecl("_d", typeof(int));
            TestClass.Add(new MultiFieldDecl(_c, _d));
            var E = new FieldDecl("E", typeof(int), Modifiers.Private | Modifiers.Const, _maxItems); TestClass.Add(E);
            var _testEnum = new FieldDecl("_testEnum", TestEnum, Modifiers.Private, new Dot(TestEnum, testEnum_Four)); TestClass.Add(_testEnum);


            // CONSTRUCTORS & DESTRUCTORS

            TestClass.Add(new Comment("CONSTRUCTORS & DESTRUCTORS"));
            TestClass.Add(
                new ConstructorDecl(Modifiers.Static)
                    {
                        Comment = "Static constructor",
                        Body =
                            {
                                new Assignment(_instance, new NewObject(TestClass.GetConstructor())),
                                new Call(new Dot(_instance, MethodRef.Find(typeof(ArrayList), "Add", typeof(object))), new Literal(null)) { EOLComment = "base method call" }
                            }
                    }
                );

            TestClass.Add(
                new DocSummary("\nDoc comment with missing language element.\n"),
                new Comment("private int field;") { NewLines = 1, NoSpaceAfterDelimiter = true }
                );

            ConstructorDecl TestClass_ctor1;
            {
                var ctor1_baseInitializer = new BaseInitializer(ConstructorRef.Find(typeof(ArrayList)));
                ctor1_baseInitializer.AttachComment("infix", AnnotationFlags.IsInfix1 | AnnotationFlags.NoSpace);
                TestClass_ctor1 = new ConstructorDecl(Modifiers.Public)
                                      {
                                          Initializer = ctor1_baseInitializer,
                                          DocComment = new DocComment(new DocSummary("Default constructor"), "\ntest")
                                      };
                TestClass.Add(TestClass_ctor1);
            }

            ConstructorDecl TestClass_ctor2;
            {
                var ctor2_param = new ParameterDecl("test", typeof(string));
                TestClass_ctor2 = new ConstructorDecl(Modifiers.Public, ctor2_param)
                                      {
                                          Initializer = new ThisInitializer(TestClass_ctor1),
                                          DocComment = new DocComment(new DocSummary("\nConstructor w/parameter.\n"), new DocParam(ctor2_param, "Test parameter."))
                                      };
                TestClass.Add(TestClass_ctor2);
            }

            var thisParameter = new Literal("test") { IsFirstOnLine = true };
            TestClass.Add(
                new ConstructorDecl(Modifiers.Public, new ParameterDecl("test", typeof(int)))
                    {
                        EOLComment = "EOL comment on ConstructorDecl",
                        Initializer = new ThisInitializer(TestClass_ctor2, thisParameter) { Comment = "Comment on ConstructorInitializer", EOLComment = "EOL comment on ConstructorInitializer" }
                    },
                new DestructorDecl { Comment = "Destructor" },
                new MethodDecl("Dispose", typeof(void), Modifiers.Public)  // IDisposable implementation
                );


            // PROPERTIES

            TestClass.Add(new Comment("PROPERTIES"));
            var _testRef = _test.CreateRef();
            _testRef.AttachAnnotation(new Comment("inline") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
            var get_Test = new GetterDecl(new Return(_testRef)) { Body = { EOLComment = "getter" } };
            var set_Test = new SetterDecl(Modifiers.Protected) { Body = { EOLComment = "setter" } };
            set_Test.AttachAnnotation(new Attribute(classCustomAttribute));
            set_Test.Add(new Assignment(_test, set_Test.ValueParameter) { EOLComment = "EOL comment" });
            var TestClass_Test = new PropertyDecl("Test", typeof(string), Modifiers.Public)
                                     {
                                         Getter = get_Test,
                                         Setter = set_Test,
                                         DocComment = new DocComment(new DocSummary("\nTest property.\n"), new DocValue("This is a test value."))
                                     };
            TestClass_Test.AttachAnnotation(new Attribute(classCustomAttribute) { EOLComment = "property attribute" });
            TestClass.Add(TestClass_Test);


            // INDEXERS

            TestClass.Add(new Comment("INDEXERS"));
            var ip1 = new ParameterDecl("p1", typeof(int));
            var ip2 = new ParameterDecl("p2", typeof(int));
            var get_Item = new GetterDecl(new Return(new Index(_2darray, ip1, ip2)));
            var set_Item = new SetterDecl(Modifiers.Private);
            set_Item.AttachAnnotation(new Attribute(classCustomAttribute));
            set_Item.Add(new Assignment(new Index(_2darray, ip1, ip2), set_Item.ValueParameter));
            var indexer = new IndexerDecl(typeof(double), Modifiers.Public, ip1, ip2)
                              {
                                  Getter = get_Item,
                                  Setter = set_Item,
                                  DocComment = new DocComment(new DocSummary("\nThis is an indexer.\n"),
                                      new DocParam(ip1, "First parameter ", new DocC(ip1), "."),
                                      new DocParam(ip2, "Second parameter ", new DocParamRef(ip2), "."),
                                      new DocReturns("Double result."))
                              };
            indexer.AttachAnnotation(new Attribute(classCustomAttribute) { EOLComment = "indexer attribute" });
            TestClass.Add(indexer);


            // Forward declarations for methods:

            var methodVoid1 = new MethodDecl("MethodVoid1", typeof(void), Modifiers.Static) { EOLComment = "No parameters, empty body, static" };
            methodVoid1.AttachComment("infix", AnnotationFlags.IsInfix1 | AnnotationFlags.NoSpace);

            MethodDecl methodVoid2;
            {
                var p1 = new ParameterDecl("p1", typeof(double));
                var p2 = new ParameterDecl("p2", typeof(double));
                methodVoid2 = new MethodDecl("MethodVoid2", typeof(void), Modifiers.New, p1, p2) { EOLComment = "Two parameters" };
                methodVoid2.Add(new Return { EOLComment = "Return with no value" });
            }

            MethodDecl methodUIntULong;
            {
                var p1 = new ParameterDecl("p1", typeof(uint));
                var p2 = new ParameterDecl("p2", typeof(ulong));
                methodUIntULong = new MethodDecl("MethodUIntULong", typeof(void), p1, p2);
            }

            MethodDecl methodString;
            {
                var p1 = new ParameterDecl("p1", typeof(string), ParameterModifier.Ref);
                var p2 = new ParameterDecl("p2", typeof(double), ParameterModifier.Out) { IsFirstOnLine = true };
                methodString = new MethodDecl("MethodString", typeof(string), p1, p2) { EOLComment = "Two parameters" };
                methodString.Add(new Assignment(p2, 0), new Return(p1));
            }

            GenericMethodDecl methodGeneric;
            {
                var U = new TypeParameter("U");
                var T = new TypeParameter("T");
                var u = new ParameterDecl("u", U);
                methodGeneric = new GenericMethodDecl("MethodGeneric", T, u) { EOLComment = "EOL comment on GenericMethodDecl" };
                methodGeneric.AttachAnnotation(new DocComment(new DocSummary("\nGeneric method.\n"), new DocTypeParam(T, "Type parameter T."),
                    new DocTypeParam(U, "Type parameter U."), new DocParam(u, "Parameter u."), new DocReturns("Result T.")));
                methodGeneric.AddTypeParameters(T, U);
                methodGeneric.AddConstraintClauses(new ConstraintClause(T, new TypeConstraint(U)) { IsFirstOnLine = true, EOLComment = "EOL comment on 1st constraint" },
                    new ConstraintClause(U, new ClassConstraint()) { IsFirstOnLine = true, EOLComment = "EOL comment on 2nd constraint" });
                methodGeneric.Add(new Return(new Conditional(new Equal(u, new Literal(null)),
                    new DefaultValue(T), new Cast(T, u))));
            }

            GenericMethodDecl methodGeneric2;
            {
                var T = new TypeParameter("T");
                var obj = new ParameterDecl("obj", T);
                methodGeneric2 = new GenericMethodDecl("MethodGeneric2", new TypeRef(T, 1), Modifiers.New, obj);
                methodGeneric2.AddTypeParameters(T);
                methodGeneric2.Add(new Return(new NewArray(T, 1)));
            }

            var console = project.Find("System.Console") as Type;

            MethodDecl methodInt;
            {
                var pa = new ParameterDecl("pa", typeof(int[]), ParameterModifier.Params);
                methodInt = new MethodDecl("MethodInt", typeof(int), pa) { EOLComment = "One parameter" };
                pa.AttachComment("inline", AnnotationFlags.NoSpace);
                methodInt.Add(new Return(new Multiply(new Index(pa, 0), -1)) { EOLComment = "Return a value" });

                var result = new LocalDecl("result", typeof(int), new Call(methodInt, 10));
                var list = new LocalDecl("list", typeof(List<int>), new NewObject(typeof(List<int>)));
                var bx = new LocalDecl("b", typeof(bool), new LessThanEqual(new BitwiseAnd(result, new Literal("0x80", true)), 3));
                var ox = new LocalDecl("o", typeof(object), new Index(list, 0));

                var ListT = new TypeParameterRef(typeof(List<>).GetGenericArguments()[0]);
                methodInt.AttachAnnotation(new DocComment(
                    new DocSummary("\nTest method with ", new DocB("one"), " parameter.\nThe ", new DocParamRef(pa), " parameter is an ", new DocI("optional"), " list of ",
                        new DocC(typeof(int)), ".\nUse standard XML escapes to embed chars such as: < & >", new DocPara("2nd paragraph"), "\n",
                        new DocSee(new Dot(typeof(string), MethodRef.Find(typeof(string), "IndexOf"))), new DocSeeAlso(new Call(methodString, new Ref(typeof(string)), new Out(typeof(double)))),
                        "\nGenerics can be referenced like ", new DocSee(new TypeRef(typeof(List<>), ListT)),
                        " or ", new DocSee(new TypeRef(typeof(List<>), ListT)), ".\n"),
                    new DocParam(pa, ""), new DocReturns("The result."),
                    new DocException(project.ParseName("System.Exception"), "Thrown when..."),
                    new DocPermission(project.ParseName("System.Security.PermissionSet"), "Everyone can access this method."),
                    new DocRemarks("\nThis is the remarks section.\n", new DocSee(new Call(new Dot(project.ParseName("System.Threading.Monitor"),
                        MethodRef.Find(typeof(Monitor), "Enter", typeof(object), typeof(bool))), typeof(object), new Ref(typeof(bool)))),
                        "\nHere is a list:", new DocList("bullet", new DocListHeader(new DocListTerm("Header Term"), new DocListDescription("Header Description")),
                            new DocListItem(new DocListTerm("Term 1"), new DocListDescription("Description 1.")), new DocListItem(new DocListTerm("Term 2"), new DocListDescription("Description 2."))),
                            "\n"),
                    new DocExample("This example shows how to call the ", new DocSee(methodInt), " method.",
                        new DocCode(new BlockDecl(
                            result, list, bx, ox,
                            new LocalDecl("i", typeof(int), new Call(methodGeneric.CreateRef(typeof(int), typeof(object)), ox)),
                            new LocalDecl("s", typeof(string), "<&>"),
                            new If(bx, new If(new GreaterThan(result, 0), new Call(new Dot(console, MethodRef.Find(console, "WriteLine", typeof(string))), "")))
                            )), "\n")
                    ));
            }

            MethodDecl methodIterator;
            {
                methodIterator = new MethodDecl("GetEnumerator", typeof(IEnumerator<int>), Modifiers.Public) { EOLComment = "Iterator" };
                methodIterator.Add(new YieldReturn(1), new YieldReturn(2), new YieldBreak());
            }

            var testStruct = new StructDecl("TestStruct", Modifiers.Public) { DocComment = new DocSummary("\nThis is a top-level struct.\n") };


            // DELEGATES

            TestClass.Add(new Comment("DELEGATES"));
            var del_VoidNoParms = new DelegateDecl("VoidNoParms", typeof(void)); TestClass.Add(del_VoidNoParms);
            var del_VoidString = new DelegateDecl("VoidString", typeof(void), Modifiers.Public, new ParameterDecl("msg", typeof(string))); TestClass.Add(del_VoidString);
            var del_VoidTwoParms = new DelegateDecl("VoidTwoParms", typeof(void), Modifiers.Public, new ParameterDecl("p1", typeof(double)),
                new ParameterDecl("p2", typeof(double))); TestClass.Add(del_VoidTwoParms);
            var del_StringTwoParms = new DelegateDecl("StringTwoParms", typeof(string), Modifiers.Public, new ParameterDecl("p1", typeof(string), ParameterModifier.Ref),
                new ParameterDecl("p2", typeof(double), ParameterModifier.Out)); TestClass.Add(del_StringTwoParms);

            var dT = new TypeParameter("T");
            var del_GenericIntParamsT = new DelegateDecl("GenericIntParamsT", typeof(int), Modifiers.Private, dT);
            del_GenericIntParamsT.AddConstraintClauses(new ConstraintClause(dT, new StructConstraint()));
            del_GenericIntParamsT.AddParameter(new ParameterDecl("p1", dT.CreateArrayRef(1), ParameterModifier.Params));
            TestClass.Add(del_GenericIntParamsT);

            var methodInfo_WriteLine = console.GetMethodExt("WriteLine");
            var propertyInfo_Length = typeof(string[]).GetProperty("Length");
            var _del1 = new FieldDecl("_del1", del_VoidNoParms, Modifiers.Private, new NewObject(del_VoidNoParms, new Dot(console, methodInfo_WriteLine))); TestClass.Add(_del1);
            var _del2 = new FieldDecl("_del2", del_VoidTwoParms, Modifiers.Private, new Dot(_instance, methodVoid2)); TestClass.Add(_del2);
            var _del3 = new FieldDecl("_del3", del_StringTwoParms, Modifiers.Private, new Dot(_instance, methodString)); TestClass.Add(_del3);
            var _del4 = new FieldDecl("_del4", del_GenericIntParamsT.CreateRef(typeof(int)), Modifiers.Private,
                new Dot(_instance, methodInt)); TestClass.Add(_del4);
            var _adel1 = new FieldDecl("_adel1", del_VoidNoParms, Modifiers.Private,
                new AnonymousMethod(new LocalDecl("i", typeof(int), 2), null)); TestClass.Add(_adel1);
            var aip1 = new ParameterDecl("ai", typeof(int[]));
            var _adel2 = new FieldDecl("_adel2", del_GenericIntParamsT.CreateRef(typeof(int)), Modifiers.Private,
                new AnonymousMethod(new Return(new Dot(aip1, propertyInfo_Length)), aip1) { IsSingleLine = false, EOLComment = "EOL comment" }); TestClass.Add(_adel2);
            var _adel3 = new FieldDecl("_adel3", del_VoidNoParms, Modifiers.Private,
                new LambdaExpression(new Call(new Dot(console, methodInfo_WriteLine), ""))); TestClass.Add(_adel3);
            var aip2 = new ParameterDecl("ai", new VarTypeRef());
            var _adel4 = new FieldDecl("_adel4", del_GenericIntParamsT.CreateRef(typeof(int)), Modifiers.Private,
                new LambdaExpression(new Dot(aip2, propertyInfo_Length), aip2)); TestClass.Add(_adel4);

            var WriteLine = new FieldDecl("WriteLine", del_VoidString, Modifiers.Public | Modifiers.Static, new NewObject(del_VoidString, new Dot(console, methodInfo_WriteLine))) { NewLines = 2 };
            TestClass.Add(WriteLine);

            var MethodWithDelegateParameter1 = new MethodDecl("MethodWithDelegateParameter", typeof(void), Modifiers.Private | Modifiers.Static);
            {
                TestClass.Add(MethodWithDelegateParameter1);
                var del = new ParameterDecl("del", del_VoidNoParms);
                MethodWithDelegateParameter1.AddParameters(new ParameterDecl("s", typeof(string)), del);
                MethodWithDelegateParameter1.Add(new Call(del));
            }
            var MethodWithDelegateParameter2 = new MethodDecl("MethodWithDelegateParameter", typeof(void), Modifiers.Private | Modifiers.Static) { NewLines = 1 };
            {
                TestClass.Add(MethodWithDelegateParameter2);
                var del = new ParameterDecl("del", del_VoidTwoParms);
                MethodWithDelegateParameter2.AddParameters(new ParameterDecl("s", typeof(string)), del);
                MethodWithDelegateParameter2.Add(new Call(del, new Literal("1.0", true), new Literal("2.0", true)));
            }
            var MethodWithDelegateParameter3 = new MethodDecl("MethodWithDelegateParameter3", typeof(string), Modifiers.Private | Modifiers.Static) { NewLines = 1 };
            {
                TestClass.Add(MethodWithDelegateParameter3);
                var ps = new ParameterDecl("s", typeof(string));
                var del = new ParameterDecl("del", del_GenericIntParamsT.CreateRef(typeof(int)));
                MethodWithDelegateParameter3.AddParameters(ps, del);
                MethodWithDelegateParameter3.Add(new Return(new Add(ps, new Call(del, 1, 2))));
            }
            var MethodWithDelegateParameter4 = new MethodDecl("MethodWithDelegateParameter4", typeof(string), Modifiers.Private | Modifiers.Static) { NewLines = 1 };
            {
                TestClass.Add(MethodWithDelegateParameter4);
                var del = new ParameterDecl("del", typeof(Func<IEnumerable<double>, string>));
                MethodWithDelegateParameter4.AddParameters(new ParameterDecl("s", typeof(string)), del);
                MethodWithDelegateParameter4.Add(new Return(new Literal(null)));
            }

            var method_Generic = new GenericMethodDecl("Generic", typeof(void), Modifiers.Public, new TypeParameter("T")) { NewLines = 1 };

            {
                var TestDelegates = new MethodDecl("TestDelegates", typeof(void), Modifiers.Public) { NewLines = 1 };
                TestClass.Add(TestDelegates);
                TestDelegates.Add(
                    new Call(MethodWithDelegateParameter1, "1a", new AnonymousMethod(new Call(new Dot(console, methodInfo_WriteLine)))),
                    new Call(MethodWithDelegateParameter1, "1b", new LambdaExpression(new Call(new Dot(console, methodInfo_WriteLine)))),
                    new Call(MethodWithDelegateParameter1, "1c", new Dot(console, methodInfo_WriteLine)),
                    new Call(MethodWithDelegateParameter1, "1d", method_Generic.CreateRef(typeof(int)))
                );

                {
                    var pa = new ParameterDecl("a", typeof(double));
                    var pb = new ParameterDecl("b", typeof(double));
                    TestDelegates.Add(new Call(MethodWithDelegateParameter2, "2a", new AnonymousMethod(new Assignment(pa, pb), pa, pb)) { NewLines = 2 });
                }
                {
                    var pa = new ParameterDecl("a", typeof(double));
                    var pb = new ParameterDecl("b", typeof(double));
                    TestDelegates.Add(new Call(MethodWithDelegateParameter2, "2b", new LambdaExpression(new Assignment(pa, pb), pa, pb) { IsSingleLine = false, HasBraces = true }));
                }
                {
                    var pa = new ParameterDecl("a", new VarTypeRef());
                    var pb = new ParameterDecl("b", new VarTypeRef());
                    TestDelegates.Add(new Call(MethodWithDelegateParameter2, "2c", new LambdaExpression(new Assignment(pa, pb), pa, pb)));
                }

                TestDelegates.Add(new Call(MethodWithDelegateParameter3, "3a", new AnonymousMethod(new Return(1), null)) { NewLines = 2 });
                {
                    var ai = new ParameterDecl("ai", typeof(int[]));
                    TestDelegates.Add(new Call(MethodWithDelegateParameter3, "3b", new AnonymousMethod(new Return(new Dot(ai, propertyInfo_Length)), ai)));
                }
                {
                    var ai = new ParameterDecl("ai", new VarTypeRef());
                    TestDelegates.Add(new Call(MethodWithDelegateParameter3, "3c", new LambdaExpression(new Dot(ai, propertyInfo_Length), ai)));
                }

                MethodInfo methodInfo_Concat = typeof(string).GetMethodExt("Concat", typeof(IEnumerable<>));
                TestDelegates.Add(new Call(MethodWithDelegateParameter4, "4a", new Dot(typeof(string), new MethodRef(methodInfo_Concat, typeof(double)))) { NewLines = 2 });

                LocalDecl dela;
                {
                    var ps1 = new ParameterDecl("s1", new VarTypeRef());
                    var ps2 = new ParameterDecl("s2", new VarTypeRef());
                    MethodInfo methodInfo_Concat_object_object = typeof(string).GetMethodExt("Concat", typeof(object), typeof(object));
                    dela = new LocalDecl("dela", typeof(Action<double, double>),
                        new LambdaExpression(new Call(new Dot(typeof(string), new MethodRef(methodInfo_Concat_object_object)), ps1, ps2), ps1, ps2)) { NewLines = 2 };
                    TestDelegates.Add(dela);
                }
                TestDelegates.Add(new Call(MethodWithDelegateParameter1, "1a", new NewObject(del_VoidTwoParms, dela)));

                {
                    var ai = new ParameterDecl("ai", new VarTypeRef());
                    TestDelegates.Add(new LocalDecl("func1", typeof(Func<int[], int>), new LambdaExpression(new Dot(ai, propertyInfo_Length), ai)) { NewLines = 2 });
                }
                LocalDecl func2;
                {
                    var ai = new ParameterDecl("ai", new VarTypeRef());
                    func2 = new LocalDecl("func2", typeof(Expression<Func<int[], int>>), new LambdaExpression(new Dot(ai, propertyInfo_Length), ai));
                    TestDelegates.Add(func2);
                }
                {
                    var methodRef_Compile = MethodRef.Find(typeof(Expression<>), "Compile", BindingFlags.Instance | BindingFlags.Public | BindingFlags.DeclaredOnly, false);
                    LocalDecl del = new LocalDecl("del", new VarTypeRef(), new Call(new Dot(func2, methodRef_Compile)));
                    TestDelegates.Add(del);
                    TestDelegates.Add(new Call(new Dot(del, MethodRef.Find(typeof(Func<,>), "Invoke", typeof(Func<,>).GetGenericArguments()[0])),
                        new NewArray(typeof(int[])) { Initializer = new Initializer(1, 2) }));
                }

                TestDelegates.Add(new LocalDecl("func1b", typeof(Func<int[], int>), methodInt) { NewLines = 2 });
            }
            TestClass.Add(method_Generic);


            // EVENTS

            TestClass.Add(new Comment("EVENTS"));
            var multiEvents = new MultiFieldDecl(del_VoidNoParms, Modifiers.Event) { NewLines = 2, EOLComment = "field-like events" };
            var _event1 = new FieldDecl("_event1", del_VoidNoParms, new Literal(null));
            multiEvents.Add(_event1);
            multiEvents.Add("_event2", methodVoid1);
            TestClass.Add(multiEvents);

            var add_Event2 = new AdderDecl { Body = { EOLComment = "adder" } };
            add_Event2.AttachAnnotation(new Attribute(classCustomAttribute));
            add_Event2.Add(new AddAssign(_event1, add_Event2.ValueParameter));
            var remove_Event2 = new RemoverDecl { Body = { EOLComment = "remover" } };
            remove_Event2.Add(new SubtractAssign(_event1, add_Event2.ValueParameter));
            var event2 = new EventDecl("Event2", del_VoidNoParms) { Adder = add_Event2, Remover = remove_Event2, Comment = "This is an event" };
            event2.AttachAnnotation(new Attribute(classCustomAttribute) { EOLComment = "event attribute" });
            TestClass.Add(event2);


            // OPERATOR OVERLOADING

            TestClass.Add(new Comment("OPERATOR OVERLOADING"));
            var op_positive = new OperatorDecl(Add.ParseToken, typeof(int), Modifiers.Public | Modifiers.Static,
                new Return(0), new ParameterDecl("p", TestClass)) { NewLines = 2, IsSingleLine = true, Body = { EOLComment = "unary" } }; TestClass.Add(op_positive);
            var op_add = new OperatorDecl(Add.ParseToken, typeof(int), Modifiers.Public | Modifiers.Static,
                new Return(0), new ParameterDecl("left", TestClass), new ParameterDecl("right", typeof(int))) { IsSingleLine = true, Body = { EOLComment = "binary" } }; TestClass.Add(op_add);
            var op_iconv = new ConversionOperatorDecl(typeof(string), Modifiers.Public | Modifiers.Static | Modifiers.Implicit,
                new Return(new Literal(null)), new ParameterDecl("p", TestClass)) { IsSingleLine = true, Body = { EOLComment = "implicit conversion" } }; TestClass.Add(op_iconv);
            var op_econv = new ConversionOperatorDecl(typeof(double), Modifiers.Public | Modifiers.Static | Modifiers.Explicit,
                new Return(0), new ParameterDecl("p", TestClass)) { IsSingleLine = true, Body = { EOLComment = "explicit conversion" } }; TestClass.Add(op_econv);
            var op_true = new BooleanOperatorDecl(True.ParseToken, Modifiers.Public | Modifiers.Static,
                new Return(true), new ParameterDecl("obj", TestClass)) { IsSingleLine = true }; TestClass.Add(op_true);
            var op_false = new BooleanOperatorDecl(False.ParseToken, Modifiers.Public | Modifiers.Static,
                new Return(false), new ParameterDecl("obj", TestClass)) { IsSingleLine = true }; TestClass.Add(op_false);

            var OperatorOverloading = new MethodDecl("OperatorOverloading", typeof(void), Modifiers.Public | Modifiers.Static);
            {
                var dateTime_Now = PropertyRef.Find(typeof(DateTime), "Now");
                var dt1 = new LocalDecl("dt1", typeof(DateTime), new Dot(typeof(DateTime), dateTime_Now));
                var diff = new LocalDecl("diff", typeof(TimeSpan), new Subtract(new Dot(typeof(DateTime), dateTime_Now), dt1).ResolveOverload());
                OperatorOverloading.Add(dt1, diff, new AddAssign(dt1, diff).ResolveOverload(), new Call(new Dot(dt1, MethodRef.Find(typeof(DateTime), "ToString"))));
                var t = new LocalDecl("t", TestClass, new NewObject(TestClass));
                var _i = new LocalDecl("i", typeof(int), new Add(t, 5).ResolveOverload());
                OperatorOverloading.Add(t, _i, new Call(new Dot(typeof(int), MethodRef.Find(typeof(int), "Parse", typeof(string))), t),
                    new LocalDecl("d", typeof(double), new Cast(typeof(double), t).ResolveOverload()));
            }
            TestClass.Add(OperatorOverloading);


            MethodDecl method_ArgList;
            {
                var argList = new ArgListParameterDecl();
                method_ArgList = new MethodDecl("ArgList", typeof(void), new ArgListParameterDecl()) { Comment = "UNDOCUMENTED __arglist keyword", NewLines = 1 };
                var ai = new LocalDecl("ai", typeof(ArgIterator), new NewObject(typeof(ArgIterator), argList)); method_ArgList.Add(ai);
                var @while = new While(new GreaterThan(new Call(new Dot(ai, MethodRef.Find(ai.EvaluateType(), "GetRemainingCount"))), 0));
                {
                    var tr = new LocalDecl("tr", typeof(TypedReference), new Call(new Dot(ai, MethodRef.Find(ai.EvaluateType(), "GetNextArg")))); @while.Add(tr);
                    @while.Add(new Call(new Dot(console, WriteLine), new Call(new Dot(typeof(TypedReference), MethodRef.Find(typeof(TypedReference), "ToObject", typeof(TypedReference))), tr)));
                }
                method_ArgList.Add(@while);
            }


            // void MethodTest()
            var MethodTest = new MethodDecl("MethodTest", typeof(void)) { Comment = "Main Testing Method" };
            {
                MethodTest.AttachAnnotation(new Attribute(new Call(ConstructorRef.Find(typeof(ConditionalAttribute), typeof(string)), "DEBUG"),
                    new Call(ConstructorRef.Find(typeof(ConditionalAttribute), typeof(string)), "TRACE") { IsFirstOnLine = true }) { EOLComment = "method attributes" });
                MethodTest.AttachAnnotation(new Attribute(ConstructorRef.Find(typeof(ObsoleteAttribute), typeof(string), typeof(bool)),
                    "Use new method", true));

                MethodTest.Add(new Comment("----- TYPES -----\n(using Variables and Literals to test)"));

                var uo = new LocalDecl("uo", typeof(object)) { NewLines = 2, EOLComment = "uninitialized" }; MethodTest.Add(uo);
                var o = new LocalDecl("o", typeof(object), new Literal(null)) { EOLComment = "null Literal" }; MethodTest.Add(o);
                var b1 = new LocalDecl("b1", typeof(bool), true) { EOLComment = "true Literal" }; MethodTest.Add(b1);
                var b2 = new LocalDecl("b2", typeof(bool), false) { EOLComment = "false Literal" }; MethodTest.Add(b2);
                var b = new LocalDecl("b", typeof(byte), 255); MethodTest.Add(b);
                var sb = new LocalDecl("sb", typeof(sbyte), -128); MethodTest.Add(sb);
                var s = new LocalDecl("s", typeof(short), -32768); MethodTest.Add(s);
                var us = new LocalDecl("us", typeof(ushort), 65535); MethodTest.Add(us);
                var i = new LocalDecl("i", typeof(int), -2147483648); MethodTest.Add(i);
                var ui = new LocalDecl("ui", typeof(uint), 4294967295); MethodTest.Add(ui);
#pragma warning disable 78
                var l = new LocalDecl("l", typeof(long), new Literal("-9223372036854775808l", true)); MethodTest.Add(l);
#pragma warning restore 78
                var ul = new LocalDecl("ul", typeof(ulong), new Literal("18446744073709551615L", true)); MethodTest.Add(ul);
                var f1 = new LocalDecl("f1", typeof(float), new Literal("3.4e38f", true)); MethodTest.Add(f1);
                var f2 = new LocalDecl("f2", typeof(float), new Literal("-1.5e-45F", true)); MethodTest.Add(f2);
                var d1 = new LocalDecl("d1", typeof(double), new Literal("1.7e308d", true)); MethodTest.Add(d1);
                var d2 = new LocalDecl("d2", typeof(double), new Literal("-5.0e-324D", true)); MethodTest.Add(d2);
                var m = new LocalDecl("m", typeof(decimal), 1000000m); MethodTest.Add(m);
                var c1 = new LocalDecl("c1", typeof(char), 'a'); MethodTest.Add(c1);
                var c2 = new LocalDecl("c2", typeof(char), new Literal(@"'\\'", true)); MethodTest.Add(c2);
                var s1 = new LocalDecl("s1", typeof(string), new Literal(null)); MethodTest.Add(s1);
                var s2 = new LocalDecl("s2", typeof(string), @"\\"); MethodTest.Add(s2);
                var s3 = new LocalDecl("s3", typeof(string), "test"); MethodTest.Add(s3);

                MethodTest.Add(new Comment("enums"));
                var e1 = new LocalDecl("e1", enumE1, new Cast(enumE1, 0)); MethodTest.Add(e1);
                var e2 = new LocalDecl("e2", enumE1, new Dot(enumE1, enumE1_two)); MethodTest.Add(e2);
                MethodTest.Add(new LocalDecl("e3", TestEnum, new BitwiseOr(new BitwiseOr(new Dot(TestEnum, testEnum_One), new Dot(TestEnum, testEnum_Two)), _testEnum)));

                MethodTest.Add(new Comment("Arrays (simple)"));
                var ia = new LocalDecl("ia", typeof(int[]),
                    new Initializer(1, 2, 3, new Add(i, 2))); MethodTest.Add(ia);
                var cha = new LocalDecl("cha", typeof(char[]), new Initializer(' ', new Literal(@"'\''", true), new Literal(@"'\""'", true),
                    new Literal(@"'\n'", true), new Literal(@"'\xffff'", true), new Cast(typeof(char), new Add(c1, 2)))); MethodTest.Add(cha);
                var sa = new LocalDecl("sa", typeof(string[]), new Initializer("one", "two", new Add(s3, "val"))); MethodTest.Add(sa);
                var new_ia2 = new NewArray(typeof(int), 2) { Initializer = new Initializer(1, 2) };
                var ia2 = new LocalDecl("ia2", typeof(int[]), new_ia2); MethodTest.Add(ia2);

                MethodTest.Add(new Comment("Multi-dimensional Arrays"));
                var dax2 = new LocalDecl("dax2", typeof(double[,]),
                    new Initializer(new Initializer(1, 2), new Initializer(3, 4))); MethodTest.Add(dax2);
                var new_dax2b = new NewArray(typeof(double), 2, 2) { Initializer = new Initializer(new Initializer(1, 2), new Initializer(3, 4)) };
                var dax2b = new LocalDecl("dax2b", typeof(double[,]), new_dax2b); MethodTest.Add(dax2b);
                var new_dax2c = new NewArray(new VarTypeRef(), null, null)
                                    {
                                        Initializer = new Initializer(new Initializer(new Literal("1.0", true), 2) { EOLComment = "Line #1" },
                                            new Initializer(3, 4) { EOLComment = "Line #2" }) { IsSingleLine = false, InfixEOLComment = "Multi-line initializer" }
                                    };
                var dax2c = new LocalDecl("dax2c", typeof(double[,]), new_dax2c); MethodTest.Add(dax2c);

                var namespace_System = project.FindNamespace("System");
                var type_Int32 = (Type)namespace_System.Find("Int32");

                MethodTest.Add(new Comment("Jagged Arrays"));
                var ijax2_1 = new NewArray(new VarTypeRef(), null) { Initializer = new Initializer(1) };
                var ijax2_2 = new NewArray(typeof(int), 2) { Initializer = new Initializer(21, 22) };
                var ijax2 = new LocalDecl("ijax2", typeof(int[][]), new NewArray(typeof(int[]), 2) { Initializer = new Initializer(ijax2_1, ijax2_2) });
                MethodTest.Add(ijax2);
                var namespace_Test = project.FindNamespace("Test");
                var namespace_TestFull = @namespace.Namespace;
                var ijau = new LocalDecl("ijau", Dot.Create(namespace_Test, namespace_TestFull, TestClass.CreateArrayRef(1, 1)),
                    new NewArray(Dot.Create(namespace_Test, namespace_TestFull, TestClass.CreateArrayRef(1)), 1));
                MethodTest.Add(ijau);
                var ijax3_1 = new NewArray(new VarTypeRef(), null, null) { Initializer = new Initializer(new Initializer(1), new Initializer(2)) };
                var ijax3 = new LocalDecl("ijax3", typeof(int[][,]), new NewArray(typeof(int[][,])) { Initializer = new Initializer(ijax3_1) });
                MethodTest.Add(ijax3);

                MethodTest.Add(new Comment("Generic Types"));
                var namespace_Collections = (Namespace)namespace_System.Find("Collections");
                var namespace_Generic = (Namespace)namespace_Collections.Find("Generic");
                var li = new LocalDecl("li", Dot.Create(namespace_System, namespace_Collections, namespace_Generic, typeof(List<int>)),
                    new NewObject(typeof(List<int>)) { Initializer = new Initializer(1, 2) }); MethodTest.Add(li);
                var lli = new LocalDecl("lli", typeof(List<List<int>>), new NewObject(typeof(List<List<int>>))); MethodTest.Add(lli);

                MethodTest.Add(new Comment("Nullable Types"));
                var ni = new LocalDecl("ni", typeof(int?), new Cast(typeof(int?), i)); MethodTest.Add(ni);
                var ndt = new LocalDecl("ndt", TypeRef.CreateNullable(new Dot(namespace_System, typeof(DateTime))), new NewObject(TypeRef.CreateNullable(new Dot(namespace_System, typeof(DateTime))))); MethodTest.Add(ndt);
                var ni2 = new LocalDecl("ni2", typeof(int?), new Literal(null)); MethodTest.Add(ni2);
                var ns = new LocalDecl("ns", TypeRef.CreateNullable(Dot.Create(namespace_Test, namespace_TestFull, testStruct)),
                    new Literal(null)); MethodTest.Add(ns);

                MethodTest.Add(new Comment("Inferred (Implicit) Types, Anonymous Types"));
                var var1 = new LocalDecl("var1", new VarTypeRef(), new NewArray(Dot.Create(namespace_System, namespace_Collections, namespace_Generic,
                    TypeRef.Create(typeof(List<>), typeof(int?[]))), 2)); MethodTest.Add(var1);
                var newObject = new NewObject(new AnonymousTypeRef()) { Initializer = new Initializer(new Assignment(new UnresolvedRef("N"), i), new Assignment(new UnresolvedRef("Text"), s1)) };
                newObject.ResolveType();  // Resolve the unresolved property references
                var var2 = new LocalDecl("var2", new VarTypeRef(), newObject); MethodTest.Add(var2);
                var anonymousType = (TypeRef)var2.Type.EvaluateType();
                MethodTest.Add(new Assignment(s1, new Dot(var2, anonymousType.GetProperty("Text"))));

                MethodTest.Add(new Comment("const"));
                var ci = new LocalDecl("ci", typeof(int), Modifiers.Const, 1); MethodTest.Add(ci);
                var cl = new LocalDecl("cl", typeof(long), Modifiers.Const, new Add(E, ci)); MethodTest.Add(cl);
                var cs1 = new LocalDecl("cs1", typeof(string), Modifiers.Const, @" This string has a \n\""linefeed\"" in it   "); MethodTest.Add(cs1);
                var cs2 = new LocalDecl("cs2", typeof(string), Modifiers.Const, new Literal(@"@""LF esc: \n, double quote: """"""", true)); MethodTest.Add(cs2);
                var cs3 = new LocalDecl("cs3", typeof(string), Modifiers.Const, @"Escapes: \\\'\""\0\a\b\f\n\r\t\v\x0a\x7f\u2425\U0010FFFF"); MethodTest.Add(cs3);
                var cs4 = new LocalDecl("cs4", typeof(string), Modifiers.Const, new Add(cs2, cs3)); MethodTest.Add(cs4);
                var test = new LocalDecl("test", typeof(string), new Add(new Add(cs4, " "), cl)); MethodTest.Add(test);

                MethodTest.Add(new Comment("Use of namespace and type aliases"));
                var win_ClipBoard = TypeRef.Find(win_alias, "Clipboard");
                MethodTest.Add(new Call(Dot.Create(win_alias, win_ClipBoard, MethodRef.Find(win_ClipBoard.EvaluateType(), "Clear"))));
                var app_Current = PropertyRef.Find(app_alias, "Current");
                MethodTest.Add(new Call(Dot.Create(app_alias, app_Current, MethodRef.Find(app_Current.EvaluateType(), "ToString"))));
                MethodTest.Add(new LocalDecl("app", new VarTypeRef(), new NewObject(app_alias)));

                MethodTest.Add(new Comment("Hex literals"),
                    new Assignment(b, new Literal("0xff", true)),
                    new Assignment(sb, new Literal("0x7f", true)),
                    new Assignment(s, new Literal("0x7FFF", true)),
                    new Assignment(us, new Literal("0xFFFF", true)),
                    new Assignment(i, new Literal("0x7fffffff", true)),
                    new Assignment(ui, new Literal("0xffffffff", true)),
                    new Assignment(l, new Literal("0x7fffffffffffffff", true)),
                    new Assignment(ul, new Literal("0xffffffffffffffff", true)),
                    new Assignment(c1, new Cast(typeof(char), new Literal("0x0041", true))));

                MethodTest.Add(new Comment("----- OPERATORS -----"));

                MethodTest.Add(new Comment("UNARY OPERATORS"),
                    new Assignment(b1, new Not(b2)) { NewLines = 2, EOLComment = "Not" },
                    new Assignment(i, new Complement(i)) { EOLComment = "Complement" },
                    new Assignment(i, new Positive(i)) { EOLComment = "Positive" },
                    new Assignment(i, new Negative(i)) { EOLComment = "Negative" },
                    new Increment(i) { EOLComment = "Increment" },
                    new Decrement(i) { EOLComment = "Decrement" },
                    new PostIncrement(i) { EOLComment = "PostIncrement" },
                    new PostDecrement(i) { EOLComment = "PostDecrement" },
                    new Assignment(b, new Cast(typeof(byte), i)) { EOLComment = "Cast" }
                    );

                MethodTest.Add(new Comment("BINARY OPERATORS"),
                    new Assignment(i, 1) { NewLines = 2, EOLComment = "Assignment" },
                    new Assignment(i, new Add(i, 1)) { EOLComment = "Add" },
                    new Assignment(i, new Subtract(i, 1)) { EOLComment = "Subtract" },
                    new Assignment(i, new Multiply(i, 1)) { EOLComment = "Multiply" },
                    new Assignment(i, new Divide(i, 1)) { EOLComment = "Divide" },
                    new Assignment(i, new Mod(i, 1)) { EOLComment = "Mod" },
                    new Assignment(i, new BitwiseAnd(i, 1)) { EOLComment = "BitwiseAnd" },
                    new Assignment(i, new BitwiseOr(i, 1)) { EOLComment = "BitwiseOr" },
                    new Assignment(i, new BitwiseXor(i, 1)) { EOLComment = "BitwiseXor" },
                    new Assignment(b1, new And(b1, b2)) { EOLComment = "And" },
                    new Assignment(b1, new Or(b1, b2)) { EOLComment = "Or" },
                    new Assignment(b1, new Equal(i, 1)) { EOLComment = "Equal" },
                    new Assignment(b1, new NotEqual(i, 1)) { EOLComment = "NotEqual" },
                    new Assignment(b1, new LessThan(i, 1)) { EOLComment = "LessThan" },
                    new Assignment(b1, new GreaterThan(i, 1)) { EOLComment = "GreaterThan" },
                    new Assignment(b1, new LessThanEqual(i, 1)) { EOLComment = "LessThanEqual" },
                    new Assignment(b1, new GreaterThanEqual(i, 1)) { EOLComment = "GreaterThanEqual" },
                    new Assignment(i, new LeftShift(i, 1)) { EOLComment = "LeftShift" },
                    new Assignment(i, new RightShift(i, 1)) { EOLComment = "RightShift" },
                    new AddAssign(i, 1) { EOLComment = "AddAssign" },
                    new SubtractAssign(i, 1) { EOLComment = "SubtractAssign" },
                    new MultiplyAssign(i, 1) { EOLComment = "MultiplyAssign" },
                    new DivideAssign(i, 1) { EOLComment = "DivideAssign" },
                    new ModAssign(i, 1) { EOLComment = "ModAssign" },
                    new BitwiseAndAssign(i, 1) { EOLComment = "BitwiseAndAssign" },
                    new BitwiseOrAssign(i, 1) { EOLComment = "BitwiseOrAssign" },
                    new BitwiseXorAssign(i, 1) { EOLComment = "BitwiseXorAssign" },
                    new LeftShiftAssign(i, 1) { EOLComment = "LeftShiftAssign" },
                    new RightShiftAssign(i, 1) { EOLComment = "RightShiftAssign" },
                    new Assignment(s1, new IfNullThen(s2, "")) { EOLComment = "IfNullThen" }
                    );

                MethodTest.Add(new Comment("OTHER OPERATORS"),
                    new Call(methodVoid1) { NewLines = 2, EOLComment = "Call (static)" },
                    new Call(new Dot(_instance, methodVoid2), 1, 2) { EOLComment = "Call (instance)" },
                    new Assignment(new Index(ia, 0), 0) { EOLComment = "Index" },
                    new Assignment(o, new Conditional(b1, new NewObject(new TypeRef(typeof(Dictionary<,>), typeof(int), typeof(string))), new Literal(null))) { EOLComment = "Conditional" },
                    new Assignment(s1, new Call(new Dot(new Cast(typeof(object), o) { HasParens = true },
                        MethodRef.Find(typeof(object), "ToString")))) { EOLComment = "Dot" },
                    new Assignment(s1, new As(o, typeof(string))) { EOLComment = "As" },
                    new Assignment(b1, new Is(o, typeof(int))) { EOLComment = "Is" },
                    new LocalDecl("t", typeof(Type), new TypeOf(typeof(Dictionary<,>))) { EOLComment = "TypeOf" },
                    new Assignment(i, new SizeOf(typeof(int))) { EOLComment = "SizeOf" },
                    new Assignment(i, new Checked(new Increment(i))) { EOLComment = "Checked" },
                    new Assignment(i, new Unchecked(new Increment(i))) { EOLComment = "Unchecked" },
                    new Assignment(i, new NewObject(typeof(int))) { EOLComment = "New simple type" },
                    new Assignment(o, new NewObject(Dot.Create(namespace_Test, namespace_TestFull, TestClass_ctor2), "test") { Initializer = new Initializer(new Assignment(TestClass_Test, s1)) }) { EOLComment = "New class" },
                    new Assignment(o, new NewObject(testStruct)) { EOLComment = "New struct" }
                    );
                {
                    var type_PlatformID = (Type)namespace_System.Find("PlatformID");
                    MethodTest.Add(new LocalDecl("p", new VarTypeRef(), Lookup.Create(codeUnit.GlobalAlias,
                        namespace_System, type_PlatformID, FieldRef.Find(type_PlatformID, "Win32NT"))) { EOLComment = "Lookup" });
                }
                {
                    var a = new ParameterDecl("a", new VarTypeRef());
                    MethodTest.Add(new LocalDecl("del", new TypeRef(del_GenericIntParamsT, typeof(int)), new LambdaExpression(new Dot(a, PropertyRef.Find(typeof(int[]), "Length")), a)) { EOLComment = "Lambda (goes to)" });
                }

                {
                    MethodTest.Add(new IfDirective(new DirectiveSymbolRef("UNDOCUMENTED")),
                        new Comment("UNDOCUMENTED OPERATORS") { NewLines = 1 });
                    var tr = new LocalDecl("tr", typeof(TypedReference), new MakeRef(i));
                    MethodTest.Add(tr,
                        new LocalDecl("type", typeof(Type), new RefType(tr)),
                        new LocalDecl("rv", typeof(int), new RefValue(tr, typeof(int))),
                        new Call(method_ArgList, new ArgList(i, b1, s1)),
                        new EndIfDirective { NewLines = 1 }
                        );
                }

                MethodTest.Add(new Comment("This can be used to test loading of private types from referenced assemblies"));
                MethodTest.Add(new Comment("System.Web.DataAccess.SqlConnectionHelper h;") { NewLines = 1, NoSpaceAfterDelimiter = true });

                MethodTest.Add(new Comment("----- EXPRESSIONS -----"));
                MethodTest.Add(new Comment("(see above for Variables & Literals, including const, null, true, false)") { NewLines = 1 });

                MethodTest.Add(new Comment("Compound Variable Declarations"));
                var mi1 = new LocalDecl("mi1", typeof(int));
                var mi2 = new LocalDecl("mi2", typeof(int), 2) { EOLComment = "Number 2" };
                var mi3 = new LocalDecl("mi3", typeof(int)) { IsFirstOnLine = true, EOLComment = "Third one" };
                MethodTest.Add(new MultiLocalDecl(mi1, mi2, mi3));

                MethodTest.Add(new Comment("Parenthesis Usage"),
                    new Assignment(b1, new Or(new And(b1, b2), new GreaterThan(i, 0))) { EOLComment = "Nested parens" },
                    new Assignment(i, new Subtract(i, new Add(1, 2))) { EOLComment = "Necessary parens" },
                    new Assignment(i, new Negative(new Add(i, 1))) { EOLComment = "Unary operator w/necessary parens" },
                    new Assignment(i, new Add(new Add(new Add(i, 1), 2), 3)) { EOLComment = "Hide unnecessary parens: ((i + 1) + 2) + 3" },
                    new Assignment(i, new Add(new Add(i, 1), new Add(2, 3))) { EOLComment = "Retain necessary parens: (i + 1) + (2 + 3)" },
                    new Assignment(i, new Add(i, new Add(1, new Add(2, 3)))) { EOLComment = "Retain necessary parens: i + (1 + (2 + 3))" }
                    );

                var block = new BlockDecl { Comment = "Stand-alone code block", Body = { InfixEOLComment = "Infix EOL comment" } };
                block.Add(new Comment("This & Base usage"),
                    new Assignment(o, new ThisRef()) { EOLComment = "this" },
                    new Call(new Dot(new ThisRef(), methodVoid2), 1, 2) { EOLComment = "this scope" },
                    new Assignment(d1, new Index(new ThisRef(), 0, 0)) { EOLComment = "this indexer" },
                    new Call(new Dot(new BaseRef(), MethodRef.Find(typeof(object), "GetType"))) { EOLComment = "base scope" },
                    new Assignment(o, new Index(new BaseRef(), 0)) { EOLComment = "base indexer" }
                    );
                MethodTest.Add(block);

                MethodTest.Add(new Comment("Multiple statements on the same line"),
                    new Assignment(s3, s1),
                    new Assignment(s1, s2) { NewLines = 0 },
                    new Assignment(s2, s3) { NewLines = 0, EOLComment = "Swap s1 and s2" }
                    );

                MethodTest.Add(new Comment("----- STATEMENTS -----"));

                MethodTest.Add(new Comment("IF"),
                    new If(new Equal(i, 1), new EmptyStatement { EOLComment = "empty" }) { Comment = "empty statement" },
                    new If(new Equal(i, 1), new Assignment(i, 2)) { Comment = "single statement" },
                    new If(new Equal(i, 2), new Assignment(i, 1)) { Comment = "single statement, same line", IsSingleLine = true, Body = { EOLComment = "single line 'if'" } }
                    );
                {
                    var @if = new If(new Equal(i, 1), new Block(new Assignment(i, 2), new Assignment(b1, true)) { InfixEOLComment = "K&R style" }) { Comment = "block" }; MethodTest.Add(@if);
                    @if.Body.IsFirstOnLine = false;
                }

                MethodTest.Add(new Comment("IF / ELSE"),
                    new If(new Equal(i, 1), new Else()) { Comment = "empty statement" },
                    new If(new Equal(i, 1), new Assignment(i, 2), new Else(new Assignment(i, 1))) { Comment = "single statement" },
                    new If(new Equal(i, 1), new Block(new Assignment(i, 2), new Assignment(b1, true)),
                        new Else(new Block(new Assignment(i, 1), new Assignment(b1, true)))) { Comment = "block" }
                    );

                MethodTest.Add(new Comment("IF / ELSE CHAINING"),
                    new If(new Equal(i, 1), new Else(new If(new Equal(i, 2), new Block(new Assignment(i, 1), new Assignment(b1, true)),
                        new Else(new If(new Equal(i, 3), new Assignment(i, 2), new Else(new Assignment(i, 0))) { EOLComment = "if EOL comment" }) { EOLComment = "else EOL comment" }) { Comment = "if comment" }) { Comment = "else comment" }) { Comment = "WITHOUT else-if chaining" },
                    new If(new Equal(i, 1), new ElseIf(new Equal(i, 2), new Block(new Assignment(i, 1), new Assignment(b1, true)),
                        new ElseIf(new Equal(i, 3), new Assignment(i, 2), new Else(new Assignment(i, 0))) { EOLComment = "else-if EOL comment" }) { Comment = "else-if comment" }) { Comment = "WITH else-if chaining" });

                MethodTest.Add(new Comment("SWITCH / CASE / DEFAULT"),
                    new Switch(i) { Comment = "empty switch" },
                    new Switch(i, new Case(1, new Block(new Assignment(b1, true), new Break()))) { Comment = "single case" }
                    );
                {
                    LocalDecl a;
                    MethodTest.Add(new Switch(i,
                        new Case(1) { EOLComment = "empty case fall-through" },
                        new Case(2, new Block(new Comment("Comment inside brace-less block"), new Assignment(b1, true), new Break())),
                        new Case(3, new Block(a = new LocalDecl("a", typeof(int), i),
                            new If(new Equal(a, 1), new Break()), new Assignment(b2, true), new Break { EOLComment = "Break outside block should get pulled inside" }) { HasBraces = true, Comment = "Comment outside block after header" }) { Comment = "This case has braces for scope" },
                        new Default(new Block(new Assignment(b1, new Assignment(b2, false)), new Break())) { Comment = "This is the default case" }) { Comment = "multiple case, default" });
                }

                {
                    MethodTest.Add(new Comment("GOTO"));
                    var label1 = new Label("Label1") { NewLines = 2, EOLComment = "first label" };
                    var label2 = new Label("Label2") { EOLComment = "another label" };
                    MethodTest.Add(label1, label2);
                    var case1 = new Case(1); case1.Add(new Goto(label1) { EOLComment = "goto label" });
                    var case2 = new Case(2); case2.Add(new Goto(new Cast(typeof(int), new Multiply(new Literal("1L", true), 2))) { EOLComment = "goto case" });
                    var @default = new Default(new Break());
                    var case3 = new Case(3); case3.Add(new Goto(@default) { EOLComment = "goto default" });
                    MethodTest.Add(new Switch(i, case1, case2, case3, @default));
                }

                MethodTest.Add(new Comment("WHILE"),
                    new While(new LessThan(i, 100)) { Comment = "null body" },
                    new While(new LessThan(i, 100), new EmptyStatement()) { Comment = "empty statement" },
                    new While(new LessThan(i, 100), new Increment(i)) { Comment = "single statement" },
                    new While(new LessThan(i, 100), new Block(new Increment(i), new Continue())) { Comment = "block" }
                    );

                MethodTest.Add(new Comment("DO / WHILE"),
                    new While(new LessThan(i, 100), true) { Comment = "empty statement" },
                    new While(new LessThan(i, 100), true, new Increment(i)) { Comment = "single statement" },
                    new While(new LessThan(i, 100), true, new Block(new Increment(i), new Assignment(b1, true))) { Comment = "block" },
                    new While(null, true, new Block(new Increment(i), new If(new GreaterThan(i, 100), new Break()) { IsSingleLine = true })) { Comment = "do infinite loop", EOLComment = "Display as 'do' in GUI, use 'while (true)' in text I/O" }
                    );

                MethodTest.Add(new Comment("FOR"));
                {
                    var @for = new For(new Assignment(i, 0), new LessThan(i, 100), null) { Comment = "null body" };
                    @for.AttachComment("infix", AnnotationFlags.IsInfix3 | AnnotationFlags.NoSpace);
                    MethodTest.Add(@for);
                    MethodTest.Add(new For(new Assignment(i, 0), new LessThan(i, 100), new Increment(i), new EmptyStatement()) { Comment = "empty statement" });
                    var i1 = new LocalDecl("i1", typeof(int), 0);
                    var i2 = new LocalDecl("i2", typeof(int), 0);
                    @for = new For(new MultiLocalDecl(i1, i2), new LessThan(i1, 100), new Increment(i1), new AddAssign(i, i1)) { Comment = "single statement" };
                    @for.Iterations.Add(new Decrement(i2));
                    MethodTest.Add(@for);
                    @for = new For(new Assignment(i, 0), new LessThan(i, 100), new Increment(i),
                        new Block(new AddAssign(l, i), new Assignment(b1, true))) { Comment = "block" };
                    @for.Initializations.Add(new Assignment(l, 0));
                    @for.Iterations.Add(new Decrement(l));
                    MethodTest.Add(@for);
                    MethodTest.Add(new While(true, new Break()) { Comment = "null parameters - convert to 'do' in GUI, 'while (true)' in text" });
                }
                {
                    LocalDecl v2, v3;
                    MethodTest.Add(new Comment("FOREACH"),
                        new ForEach(new LocalDecl("v", typeof(int)), ia) { Comment = "empty statement" },
                        new ForEach(v2 = new LocalDecl("v", typeof(int)), ia, new AddAssign(i, v2)) { Comment = "single statement" },
                        new ForEach(v3 = new LocalDecl("v", typeof(int)), ia, new Block(new AddAssign(i, v3), new Assignment(b1, true))) { Comment = "block" }
                        );
                }

                MethodTest.Add(new Comment("----- EXCEPTIONS -----"));
                {
                    var @finally = new Finally(new Assignment(b2, true)) { EOLComment = "finally" };
                    var catch1 = new Catch(typeof(OverflowException), new Throw { EOLComment = "re-throw" }) { EOLComment = "catch w/o variable" };
                    var ex = new LocalDecl("ex", typeof(StackOverflowException));
                    var catch2 = new Catch(ex, new Throw(ex) { EOLComment = "new throw" }) { EOLComment = "catch w/variable" };
                    var catch3 = new Catch { EOLComment = "catch all exceptions" };
                    MethodTest.Add(new Try(new Increment(i), @finally, catch1, catch2, catch3) { NewLines = 2, EOLComment = "try" });
                }

                MethodTest.Add(new Using(_instance) { HasBraces = true, EOLComment = "expression", NewLines = 2 });
                MethodTest.Add(new Using(new LocalDecl("obj", Dot.Create(namespace_Test, namespace_TestFull, TestClass),
                    new NewObject(TestClass_ctor1))) { HasBraces = true, EOLComment = "local variable" });
                {
                    var obj1 = new LocalDecl("obj1", TestClass, new NewObject(TestClass_ctor1));
                    var obj2 = new LocalDecl("obj2", TestClass, new NewObject(TestClass_ctor1));
                    MethodTest.Add(new Using(new MultiLocalDecl(obj1, obj2)) { HasBraces = true, EOLComment = "multiple variables" });
                }
                {
                    var obj3 = new Using(new LocalDecl("obj3", TestClass, new NewObject(TestClass_ctor1))) { HasBraces = true, EOLComment = " and only have braces on the inner-most block)" };
                    var obj2 = new Using(new LocalDecl("obj2", TestClass, new NewObject(TestClass_ctor1)), obj3) { EOLComment = "(should display at the same indent level," };
                    MethodTest.Add(new Using(new LocalDecl("obj1", TestClass, new NewObject(TestClass_ctor1)), obj2) { EOLComment = "multiple nested usings" });
                }

                MethodTest.Add(new Comment("----- LOCK -----"),
                    new Lock(o) { Comment = "empty statement" },
                    new Lock(o, new Increment(i)) { Comment = "single statement" },
                    new Lock(o, new Block(new Increment(i), new Assignment(b1, true))) { Comment = "block" }
                    );

                MethodTest.Add(new Comment("----- METHODS -----"),
                    new Assignment(i, new Call(methodInt, 1)) { NewLines = 2 },
                    new If(new Equal(new Call(methodString, new Ref(s3), new Out(d1)), new Literal(null)),
                        new Assignment(i, new Call(methodGeneric.CreateRef(typeof(int), typeof(object)), o))),
                    new Assignment(ia, new Call(methodGeneric2, i)),
                    new Call(methodUIntULong, new Add(new Literal("1u", true), 1), new Subtract(new Literal("1ul", true), 1)),
                    new Call(methodUIntULong, new Add(ui, 1), new Subtract(ul, 1))
                    );

                MethodTest.Add(new Call(new Dot(typeof(decimal), MethodRef.Find(typeof(decimal), "Round", typeof(decimal))), new Conditional(b1, ci, new Literal("100m", true))) { NewLines = 2 });
                var string_Format = MethodRef.Find(typeof(string), "Format", typeof(string), typeof(object[]));
                var int32_MaxValue = FieldRef.Find(type_Int32, "MaxValue");
                MethodTest.Add(
                    new Call(new Dot(typeof(string), string_Format), "{0} {1}", 1, new Dot(type_Int32, int32_MaxValue)),
                    new Call(new Dot(typeof(string), string_Format), "{0} {1} {2} {3}", 1, new Dot(type_Int32, int32_MaxValue),
                        new Literal("2d", true), new Literal("3f", true)),
                    new Call(new Dot(typeof(string), string_Format), new Literal(@"""""", true)),
                    new Call(new Dot(li, MethodRef.Find(typeof(List<int>), "Add", typeof(int))), 1),
                    new Assignment(s1, new Call(new Dot(s2, MethodRef.Find(typeof(StringUtil), "NotNull", typeof(string)))))
                    );

                MethodTest.Add(
                    new Call(_del1) { NewLines = 2 },
                    new Assignment(s1, new Call(_del3, new Ref(s2), new Out(d1))),
                    new Assignment(i, new Call(_del4, 1)),
                    new AddAssign(_del1, methodVoid1)
                    );

                MethodTest.Add(new Call(new Dot(typeof(string), string_Format), new NamedArgument(ParameterRef.Find(string_Format, "args"), i),
                    new NamedArgument(ParameterRef.Find(string_Format, "format"), "{0}")) { NewLines = 2 });
                {
                    var array_Resize = MethodRef.Find(typeof(Array), "Resize", typeof(TypeUtil.T).MakeArrayType(1), typeof(int));
                    MethodTest.Add(new Call(new Dot(typeof(Array), array_Resize), new NamedArgument(ParameterRef.Find(array_Resize, "newSize"), 10),
                        new NamedArgument(ParameterRef.Find(array_Resize, "array"), new Ref(ia))));
                }
                {
                    var int_TryParse = MethodRef.Find(typeof(int), "TryParse", typeof(string), typeof(int));
                    MethodTest.Add(new Call(new Dot(typeof(int), int_TryParse), new NamedArgument(ParameterRef.Find(int_TryParse, "result"), new Out(i)),
                        new NamedArgument(ParameterRef.Find(int_TryParse, "s"), "1")));
                }
                MethodTest.Add(new Call(new Dot(ia, MethodRef.Find(typeof(Enumerable), "Any", typeof(IEnumerable<>)))));
                {
                    var enumerable_Any = MethodRef.Find(typeof(Enumerable), "Any", typeof(IEnumerable<>));
                    MethodTest.Add(new Call(new Dot(typeof(Enumerable), enumerable_Any), new NamedArgument(ParameterRef.Find(enumerable_Any, "source"), ia)));
                }

                MethodTest.Add(new Comment("----- PROPERTIES -----"),
                    new Assignment(s1, TestClass_Test) { NewLines = 2 },
                    new Assignment(TestClass_Test, s1)
                    );

                MethodTest.Add(new Comment("----- INDEXERS -----"),
                    new Assignment(d1, new Index(_instance, 0, 0)) { NewLines = 2 },
                    new Assignment(new Index(_instance, 0, 0), d1)
                    );

                MethodTest.Add(new Comment("----- EVENTS -----"),
                    new AddAssign(event2, methodVoid1) { NewLines = 2 }
                    );

                MethodTest.Add(new Comment("----- COMMENTS -----"));
                MethodTest.Add(new Comment("This is an independent comment that is split...\n\nacross 3 lines."));
                {
                    var assign = new Assignment(i, 1) { IsFirstOnLine = true, EOLComment = "This EOL comment is associated with the statement on the left", NewLines = 2 };
                    assign.AttachComment("This comment is associated with the statement below");
                    assign.AttachComment("This is a separate comment associated with the statement below, and\nit spans two lines.");
                    MethodTest.Add(assign);
                    var and = new And(b1, b2) { EOLComment = "embedded comment" };
                    MethodTest.Add(new Assignment(b1, new Or(and, new Not(new GreaterThan(i, 0)))) { NewLines = 2, EOLComment = "EOL comment" });
                }

                {
                    var @if = new If(new And(new And(new NotEqual(s, new Literal(null)) { IsFirstOnLine = true, EOLComment = "EOL on !=" },
                        new GreaterThan(new Dot(new ThisRef(), PropertyRef.Find(TestClass, "Capacity")), 0)) { IsFirstOnLine = true, EOLComment = "EOL on '&&' #1" },
                        new Dot(new Dot(namespace_System, console), PropertyRef.Find(console, "CursorVisible")) { PostfixComment = "Postfix comment", EOLComment = "EOL on '&&' #2" })) { IsEndFirstOnLine = true, NewLines = 2 };

                    var or = new Or(
                        new And(new LocalRef(b1) { IsFirstOnLine = true, EOLComment = "EOL on b1" },
                            new GreaterThan(new Cast(typeof(long), Dot.Create(namespace_System, console, PropertyRef.Find(console, "BufferWidth"))) { IsFirstOnLine = true, EOLComment = "EOL on cast" }, new Literal(0)) { EOLComment = "EOL on '>'", PostfixComment = "Postfix comment on '>'" }
                                ) { IsFirstOnLine = true, IsEndFirstOnLine = true, EOLComment = "EOL on '&&'" },
                        new Not(new LessThan(i, new Literal(0) { EOLComment = "EOL on '0'" }) { IsEndFirstOnLine = true }) { IsFirstOnLine = true, EOLComment = "EOL on '!'" }) { IsEndFirstOnLine = true };
                    or.AttachComment("EOL on ||", AnnotationFlags.IsInfix1, CommentFlags.EOL);
                    var assign = new Assignment(b1, or);

                    var objs = new LocalDecl("objs", new VarTypeRef(), new NewArray(typeof(object[]))
                        {
                            Initializer = new Initializer(new Dot(new ThisRef(), PropertyRef.Find(TestClass, "Capacity")) { EOLComment = "1st" },
                                new Cast(typeof(long), new Dot(console, PropertyRef.Find(console, "BufferWidth"))) { EOLComment = "2nd" },
                                new Or(new And(b1, b2), new Equal(i, 0)) { EOLComment = "3rd" },
                                new LocalRef(s) { EOLComment = "4th" }) { IsSingleLine = false }
                        });

                    var multi = new MultiLocalDecl(
                        new LocalDecl("i1", typeof(int), 1) { EOLComment = "1" },
                        new LocalDecl("i2", typeof(int), 2) { IsFirstOnLine = true, EOLComment = "2" },
                        new LocalDecl("i3", typeof(int), new Cast(typeof(int), new Dot(console, PropertyRef.Find(console, "BufferWidth")))) { IsFirstOnLine = true, EOLComment = "3" });

                    @if.Add(assign, objs, multi);
                    MethodTest.Add(@if);
                }


                MethodTest.Add(new Comment("There is a triple blank line after this comment..."));
                MethodTest.Add(new Comment("Block comment Line 1\n   Line 2", CommentFlags.Block) { NewLines = 4 });
                MethodTest.Add(new Comment("Block comment Line 1\n   Line 2\n   Line 3\n", CommentFlags.Block) { NewLines = 1 });

                MethodTest.Add(new Comment("TODO: This is a 'to-do' comment", CommentFlags.TODO));


                MethodTest.Add(new Comment("----- COMPILER DIRECTIVES -----"),
                    new RegionDirective("/* COMPILER DIRECTIVES */"),
                    new Comment("See top of file for #define/#undef examples, since they can only legally appear there"),
                    new IfDirective(new Or(new Or(new And(new And(new DirectiveSymbolRef("TEST1"),
                        new DirectiveSymbolRef("TEST2")), new DirectiveSymbolRef("TEST3")),
                        new Not(new DirectiveSymbolRef("TEST4"))), new And(new Equal(new DirectiveSymbolRef("TEST5"), true),
                            new NotEqual(new DirectiveSymbolRef("TEST6"), false)))) { NewLines = 2, EOLComment = "comment" },
                    new IfDirective(new DirectiveSymbolRef("TEST1"))
                        {
                            NewLines = 2,
                            SkippedText = "// This 'code' will not be compiled:\nIt's not actually code, but that doesn't matter.",
                            EOLComment = "conditional compilation"
                        },
                    new ElIfDirective(new DirectiveSymbolRef("TEST2")) { SkippedText = "This text won't be compiled.", EOLComment = "elif" },
                    new ElseDirective { EOLComment = "else" },
                    new WarningDirective("Warning message!") { NewLines = 2 },
                    //new ErrorDirective("Error message!"),
                    new Comment("#error Error message!") { HasNoIndentation = true, NoSpaceAfterDelimiter = true, NewLines = 1 },
                    new LineDirective(999, @"""C:\path\test.cs""") { EOLComment = "comment" },
                    new LineDirective(LineDirectiveType.Hidden) { EOLComment = "comment" },
                    new LineDirective(LineDirectiveType.Default) { EOLComment = "comment" },
                    new EndIfDirective { EOLComment = "endif", NewLines = 2 },
                    new EndIfDirective { NewLines = 2 }
                    );

                {
                    var one = new Literal("one") { IsFirstOnLine = true, Comment = "Regular comment" };
                    var two = new Literal("two") { IsFirstOnLine = true, Comment = "Regular comment", EOLComment = "EOL comment" };
                    two.AttachAnnotation(new ElseDirective(), true);
                    two.AttachAnnotation(new IfDirective(false) { SkippedText = "\"too\"," }, true);
                    two.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsPostfix);
                    two.AttachAnnotation(new IfDirective(false) { SkippedText = "\"three\"" }, AnnotationFlags.IsPostfix
                        );
                    two.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsPostfix);
                    MethodTest.Add(new LocalDecl("cds", typeof(string[]), new Initializer(one, two) { IsSingleLine = false, InfixEOLComment = "EOL comment" }) { NewLines = 2 });
                }

                MethodTest.Add(
                    new PragmaWarningDirective(PragmaWarningAction.Disable) { NewLines = 2, EOLComment = "comment" },
                    new PragmaWarningDirective(PragmaWarningAction.Restore) { EOLComment = "comment" },
                    new PragmaWarningDirective(PragmaWarningAction.Disable, 78, 219, 1691) { EOLComment = "comment" },
                    new PragmaWarningDirective(PragmaWarningAction.Restore, 78, 219, 1691) { EOLComment = "comment" },
                    new PragmaWarningDirective(PragmaWarningAction.Disable),
                    new PragmaChecksumDirective("\"test.cs\"", "\"{00000000-0000-0000-0000-000000000000}\"", "\"01234567\"") { NewLines = 2, EOLComment = "comment" },
                    new EndRegionDirective("/* COMPILER DIRECTIVES */")
                    );
            }
            TestClass.Add(MethodTest);


            TestClass.Add(new Comment("----- OTHER METHODS -----"),
                methodVoid1, methodVoid2, methodUIntULong, methodInt, methodString, methodGeneric, methodGeneric2, methodIterator);


            var removeDirectory = new MethodDecl("RemoveDirectory", typeof(bool), Modifiers.Static | Modifiers.Extern,
                (CodeObject)null, new ParameterDecl("name", typeof(string))) { EOLComment = "extern - no body" };
            var dll_attr = new Attribute(ConstructorRef.Find(typeof(DllImportAttribute), typeof(string)), "kernel32",
                new Assignment(FieldRef.Find(typeof(DllImportAttribute), "SetLastError"), true), new Assignment(FieldRef.Find(typeof(DllImportAttribute), "ExactSpelling"), true));
            removeDirectory.AttachAnnotation(dll_attr);
            TestClass.Add(removeDirectory);


            {
                var px = new ParameterDecl("x", typeof(double), ParameterModifier.Ref);
                var aMethod = new MethodDecl("AMethod", typeof(void), px) { EOLComment = "parameter attributes" };
                px.AttachAnnotation(new Attribute(typeof(InAttribute), typeof(OutAttribute)));
                TestClass.Add(aMethod);
            }


            TestClass.Add(new IfDirective(new DirectiveSymbolRef("UNDOCUMENTED")),
                method_ArgList,
                new EndIfDirective { NewLines = 1 });


            // NESTED, GENERIC INTERFACE
            InterfaceDecl interface1;
            MethodDecl interface1_Method;
            PropertyDecl interface1_Property;
            IndexerDecl interface1_Indexer;
            EventDecl interface1_Event;
            {
                var T = new TypeParameter("T");
                interface1 = new InterfaceDecl("InnerInterface", Modifiers.None, T) { Comment = "This is a nested, generic interface" };
                interface1_Method = new MethodDecl("Method", typeof(void), (CodeObject)null);
                interface1.Add(interface1_Method);
                interface1_Property = new PropertyDecl("Property", typeof(string)) { NewLines = 1, IsAutomatic = true };
                interface1.Add(interface1_Property);
                interface1_Indexer = new IndexerDecl(T, new ParameterDecl("i", T)) { NewLines = 1, Getter = new GetterDecl(null) };
                interface1.Add(interface1_Indexer);
                interface1_Event = new EventDecl("Event", del_VoidNoParms) { NewLines = 1 };
                interface1.Add(interface1_Event);
                TestClass.Add(interface1);
            }

            // NESTED, GENERIC CLASS
            ClassDecl InnerClass;
            {
                var T = new TypeParameter("T");
                var U = new TypeParameter("U");
                InnerClass = new ClassDecl("InnerClass", Modifiers.None, T, U)
                {
                    DocComment = new DocComment(new DocSummary("\nThis is a nested, generic class.\n"), new DocTypeParam(T, "Type parameter T."), new DocTypeParam(U, "Type parameter U.")),
                    EOLComment = "EOL comment on ClassDecl"
                };
                InnerClass.AddBaseTypes(new TypeRef(interface1, T) { IsFirstOnLine = true, Comment = "Comment on base-type list", EOLComment = "EOL comment on base type/interface" });
                InnerClass.AddConstraintClauses(new ConstraintClause(T, new StructConstraint()) { IsFirstOnLine = true, Comment = "Comment on constraints" },
                    new ConstraintClause(U, new TypeConstraint(TestClass), new NewConstraint { EOLComment = "EOL comment on constraints" }));
                InnerClass.Add(new FieldDecl("t", T, Modifiers.Protected));
                InnerClass.Add(new FieldDecl("u", U, Modifiers.Public));
                var c1_p = new FieldDecl("p", interface1.CreateRef(T), Modifiers.Public); InnerClass.Add(c1_p);
                var c1_instance = new FieldDecl("_instance", InnerClass.CreateRef(typeof(int), TestClass), Modifiers.Private | Modifiers.Static | Modifiers.ReadOnly); InnerClass.Add(c1_instance);

                ClassDecl c1_Inner2;
                GenericMethodDecl c1_Inner2Generic;
                {
                    var V = new TypeParameter("V");
                    c1_Inner2 = new ClassDecl("Inner2", Modifiers.Private, V) { DocComment = new DocComment(new DocSummary("\nNested, generic type, with generic method referencing parent type parameters.\n"), new DocTypeParam(V, "Type parameter V.")) };
                    c1_Inner2.AddConstraintClauses(new ConstraintClause(V, new TypeConstraint(typeof(IComparable))));
                    var W = new TypeParameter("W");
                    c1_Inner2Generic = new GenericMethodDecl("Inner2Generic", typeof(string), Modifiers.Public, W);
                    c1_Inner2Generic.AddConstraintClauses(new ConstraintClause(W, new TypeConstraint(U)));
                    {
                        var t = new ParameterDecl("t", T);
                        var u = new ParameterDecl("u", U);
                        var v = new ParameterDecl("v", V);
                        var w = new ParameterDecl("w", W);
                        c1_Inner2Generic.AddParameters(t, u, v, w);
                        c1_Inner2Generic.Add(new Return(new Add(new Add(new Add(new Call(new Dot(t, MethodRef.Find(typeof(object), "ToString"))), u), v), w)));
                        c1_Inner2Generic.DocComment = new DocComment(new DocSummary("\nInner2Generic test method.\nThis method has generic parameter ", new DocTypeParamRef(W), ".\n"),
                            new DocTypeParam(W, "Type parameter W."), new DocParam(t, "Parameter t."), new DocParam(u, "Parameter u."), new DocParam(v, "Parameter v."), new DocParam(w, "Parameter w."),
                            new DocReturns("String result."));
                    }
                }
                c1_Inner2.Add(c1_Inner2Generic);

                var c1_inner2 = new FieldDecl("_inner2", c1_Inner2.CreateRef(typeof(long)), Modifiers.Private); InnerClass.Add(c1_inner2);

                InnerClass.Add(new ConstructorDecl(Modifiers.Static)
                {
                    Comment = "Static constructor",
                    Body =
                        {
                            new Assignment(c1_instance, new NewObject(InnerClass.CreateRef(typeof(int), TestClass))),
                            new LocalDecl("t1", typeof(Type), new Call(new Dot(new Cast(interface1, c1_instance), MethodRef.Find(c1_instance.EvaluateType(), "GetType")))),
                            new LocalDecl("t2", typeof(Type), new Call(new Dot(new Cast(typeof(IList), new NewObject(new TypeRef(typeof(List<>), T))), MethodRef.Find(c1_instance.EvaluateType(), "GetType")))),
                            new Assignment(new Dot(c1_instance, FieldRef.Find(c1_instance.EvaluateType(), "_inner2")), new NewObject(new Dot(new TypeRef(InnerClass, typeof(int), TestClass), new TypeRef(c1_Inner2, typeof(long))))),
                            new LocalDecl("s", typeof(string), new Call(Dot.Create(c1_instance, c1_inner2, c1_Inner2Generic), 1, new Dot(TestClass, _instance), new Literal("2L", true), new Dot(TestClass, _instance)))
                        }
                });

                InnerClass.Add(new Comment("Explicit interface implementations"));
                InnerClass.Add(new MethodDecl(new Dot(interface1.CreateRef(T), interface1_Method), typeof(void)) { IsSingleLine = true });
                InnerClass.Add(new PropertyDecl(new Dot(interface1.CreateRef(T), interface1_Property), typeof(string)) { NewLines = 1, Getter = new GetterDecl(new Return("")), Setter = new SetterDecl(), IsSingleLine = true });
                InnerClass.Add(new IndexerDecl(new Dot(interface1.CreateRef(T), interface1_Indexer), T, new ParameterDecl("i", T)) { NewLines = 1, Getter = new GetterDecl(new Return(new DefaultValue(T))), IsSingleLine = true });
                InnerClass.Add(new EventDecl(new Dot(interface1.CreateRef(T), interface1_Event), del_VoidNoParms) { NewLines = 1, Adder = new AdderDecl(), Remover = new RemoverDecl(), IsSingleLine = true });

                InnerClass.Add(c1_Inner2);

                TestClass.Add(InnerClass);
            }


            // public struct InnerStruct<T>
            {
                var typeParameterT = new TypeParameter("T");
                var struct1 = new StructDecl("InnerStruct", Modifiers.Public, typeParameterT);
                struct1.AttachAnnotation(new DocComment(
                    new DocSummary("\nThis is a nested struct, generic on ", new DocTypeParamRef(typeParameterT), ".\n"),
                    new DocTypeParam(typeParameterT, "Type parameter.")));
                TestClass.Add(struct1);
            }


            // class InnerDerived : InnerClass<DateTime, TestClass>
            var InnerDerived = new ClassDecl("InnerDerived", Modifiers.None, new TypeRef(InnerClass, typeof(DateTime), TestClass));
            var inner_t = FieldRef.Find(InnerClass, "t");
            inner_t.Parent = InnerDerived;  // Set parent temporarily so EvaluateType() below can work
            InnerDerived.Add(new MethodDecl("Method", typeof(void), new LocalDecl("v", new VarTypeRef(), new Dot(inner_t, PropertyRef.Find(inner_t.EvaluateType(), "Hour")))));
            TestClass.Add(InnerDerived);


            // class TestCollection<TItem> : List<TItem>
            ClassDecl TestCollection;
            {
                var TItem = new TypeParameter("TItem");
                TestCollection = new ClassDecl("TestCollection", Modifiers.None, TItem) { IsEndFirstOnLine = false };
                TestCollection.AddBaseTypes(new TypeRef(typeof(List<>), TItem));
                TestClass.Add(TestCollection);
            }
            var CollectionMethod = new MethodDecl("CollectionMethod", typeof(void), new ParameterDecl("list", new TypeRef(typeof(List<>), TestClass))) { NewLines = 1 };
            TestClass.Add(CollectionMethod);
            var testGenericCollection = new MethodDecl("TestGenericCollection", typeof(void)) { NewLines = 1 };
            var collection = new LocalDecl("collection", new VarTypeRef(), new NewObject(new TypeRef(TestCollection, TestClass)));
            testGenericCollection.Add(collection, new Call(CollectionMethod, collection));
            TestClass.Add(testGenericCollection);


            // static Z F<X, Y, Z>(X value, Func<X, Y> f1, Func<Y, Z> f2)
            GenericMethodDecl F;
            {
                var X = new TypeParameter("X");
                var Y = new TypeParameter("Y");
                var Z = new TypeParameter("Z");
                F = new GenericMethodDecl("F", Z, Modifiers.Static, X, Y, Z) { Comment = "C# 4.0 section 7.5.2 Type Inference examples" };
                var @value = new ParameterDecl("value", X);
                var f1 = new ParameterDecl("f1", new TypeRef(typeof(Func<,>), X, Y));
                var f2 = new ParameterDecl("f2", new TypeRef(typeof(Func<,>), Y, Z));
                F.AddParameters(@value, f1, f2);
                F.Add(new Return(new Call(f2, new Call(f1, @value))));
                TestClass.Add(F);
            }
            MethodDecl F2;
            {
                F2 = new MethodDecl("F2", typeof(void), Modifiers.Public) { NewLines = 1 };

                {
                    var s = new ParameterDecl("s", new VarTypeRef());
                    var t = new ParameterDecl("t", new VarTypeRef());
                    var seconds = new LocalDecl("seconds", typeof(double), new Call(F, "1:15:30", new LambdaExpression(new Call(new Dot(typeof(TimeSpan),
                        MethodRef.Find(typeof(TimeSpan), "Parse", typeof(string))), s), s), new LambdaExpression(new Dot(t, PropertyRef.Find(typeof(TimeSpan), "TotalSeconds")), t)));
                    F2.Add(seconds);
                }

                var testClasses = new LocalDecl("testClasses", new TypeRef(TestClass, 1), new NewArray(TestClass, 1)) { NewLines = 2 }; F2.Add(testClasses);
                {
                    var t = new ParameterDecl("t", new VarTypeRef());
                    F2.Add(new Call(new Dot(testClasses, MethodRef.Find(typeof(Enumerable), "First", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, bool>))),
                        new LambdaExpression(new Equal(new Dot(t, PropertyRef.Find(TestClass, "Test")), ""), t)));
                }

                var tests = new LocalDecl("tests", new VarTypeRef(), new NewObject(new TypeRef(typeof(Dictionary<,>), typeof(int), TestClass))) { NewLines = 2 }; F2.Add(tests);
                {
                    var t = new ParameterDecl("t", new VarTypeRef());
                    var t_Type = new TypeRef(typeof(KeyValuePair<,>), typeof(int), TestClass);
                    var tests2 = new LocalDecl("tests2", new TypeRef(typeof(IEnumerable<>), t_Type),
                        new Call(new Dot(tests, MethodRef.Find(typeof(Enumerable), "Where", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, bool>))),
                            new LambdaExpression(new Call(new Dot(new Dot(new Dot(t, PropertyRef.Find(t_Type, "Value")), TestClass_Test), MethodRef.Find(typeof(string), "StartsWith", typeof(string))), "#"), t)));
                    F2.Add(tests2);
                }
                {
                    var t = new ParameterDecl("t", new VarTypeRef());
                    var maxCount = new LocalDecl("maxCount", typeof(int), new Call(new Dot(new Dot(tests, PropertyRef.Find(tests.EvaluateType(), "Values")),
                        MethodRef.Find(typeof(Enumerable), "Max", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, int>))), new LambdaExpression(new Dot(t, PropertyRef.Find(TestClass, "Count")), t)));
                    F2.Add(maxCount);
                }

                var dict2 = new LocalDecl("dict2", new VarTypeRef(), new NewObject(new TypeRef(typeof(Dictionary<,>), typeof(int), TestClass))) { NewLines = 2 }; F2.Add(dict2);
                var e = new LocalDecl("e", new Dot(new TypeRef(typeof(Dictionary<,>), typeof(int), TestClass), typeof(Dictionary<,>.Enumerator)),
                    new Call(new Dot(tests, MethodRef.Find(tests.EvaluateType(), "GetEnumerator")))); F2.Add(e);
                F2.Add(new While(new Call(new Dot(e, MethodRef.Find(e.EvaluateType(), "MoveNext"))),
                    new Call(new Dot(dict2, MethodRef.Find(dict2.EvaluateType(), "Add", typeof(TypeUtil.T), typeof(TypeUtil.T))),
                        new Dot(new Dot(e, PropertyRef.Find(e.EvaluateType(), "Current")), PropertyRef.Find(typeof(KeyValuePair<,>), "Key")),
                        new Dot(new Dot(e, PropertyRef.Find(e.EvaluateType(), "Current")), PropertyRef.Find(typeof(KeyValuePair<,>), "Value"))
                        )));

                var item = new LocalDecl("item", new VarTypeRef());
                F2.Add(new ForEach(item, tests, new Call(new Dot(new Dot(item, PropertyRef.Find(typeof(KeyValuePair<,>), "Value")),
                    MethodRef.Find(TestClass, "Generic").MakeGenericRef(typeof(int))))) { NewLines = 2 });

                TestClass.Add(F2);
            }


            // public class TestBase<T>
            ClassDecl TestBaseT;
            PropertyDecl TestBaseT_Property;
            {
                var T = new TypeParameter("T");
                TestBaseT = new ClassDecl("TestBase", Modifiers.Public, T);
                var t = new FieldDecl("t", T, Modifiers.Private, new DefaultValue(T));
                TestBaseT_Property = new PropertyDecl("Property", T, Modifiers.Public) { Getter = new GetterDecl(new Return(t)), IsSingleLine = true, NewLines = 1 };
                TestBaseT.Add(t, TestBaseT_Property);
                TestClass.Add(TestBaseT);
            }
            // public class InnerTest : TestBase<TestClass>
            ClassDecl InnerTest;
            {
                InnerTest = new ClassDecl("InnerTest", Modifiers.Public, TestBaseT.CreateRef(TestClass)) { NewLines = 1 };
                var Test = new MethodDecl("Test", typeof(void), Modifiers.Public, new LocalDecl("s1", typeof(string), new Dot(TestBaseT_Property, TestClass_Test)));
                InnerTest.Add(Test);
                TestClass.Add(InnerTest);
            }


            // public interface ITestInterface : IComparable
            @namespace.Add(new InterfaceDecl("ITestInterface", Modifiers.Public, typeof(IComparable)) { Comment = "This is a top-level interface" });


            // public struct TestStruct
            //var @struct = new StructDecl("TestStruct", Modifiers.Public);  // Moved up for forward reference
            testStruct.Add(new PropertyDecl("Num", typeof(int), Modifiers.Public) { IsAutomatic = true, Comment = "Automatic property" });
            @namespace.Add(testStruct);


            // public static class Extension
            var s_class = new ClassDecl("Extension", Modifiers.Public | Modifiers.Static) { Comment = "This is a top-level static class" };
            MethodDecl methodExt;
            {
                var p1 = new ParameterDecl("thisStr", typeof(string), ParameterModifier.This);
                methodExt = new MethodDecl("NotNull", typeof(string), Modifiers.Public | Modifiers.Static, p1) { EOLComment = "Extension method" };
                methodExt.Add(new Return(new IfNullThen(p1, "")));
            }
            s_class.Add(methodExt);
            @namespace.Add(s_class);


            // public unsafe class Unsafe
            ClassDecl Unsafe = new ClassDecl("Unsafe", Modifiers.Public | Modifiers.Unsafe) { NewLines = 2 };
            {
                Unsafe.AttachAnnotation(new IfDirective(true));
                Unsafe.AttachAnnotation(new ElseDirective { SkippedText = "class Unsafe" }, AnnotationFlags.IsPostfix);
                Unsafe.AttachAnnotation(new EndIfDirective(), AnnotationFlags.IsPostfix);
                var i = new FieldDecl("i", typeof(int), Modifiers.Unsafe);
                var s = new FieldDecl("s", typeof(string), Modifiers.Const, "ab");
                var ts = new FieldDecl("ts", testStruct);
                var unsafeTest = new MethodDecl("Test", typeof(void), Modifiers.Unsafe);
                Unsafe.Add(i, s, ts, unsafeTest);
                var buffer = new LocalDecl("buffer", typeof(int*), new StackAlloc(typeof(int), 16));
                var p = new LocalDecl("p", typeof(int*), new AddressOf(new Index(buffer, 1)));
                var n = new LocalDecl("n", typeof(int), new SizeOf(testStruct));
                var unsafeBlock = new Unsafe();
                unsafeTest.Add(unsafeBlock);
                unsafeBlock.Add(buffer, p, n);
                {
                    var p1 = new LocalDecl("p1", typeof(int*), new AddressOf(i));
                    unsafeBlock.Add(new Fixed(p1, new Assignment(new PointerIndirection(p1), 1)) { IsSingleLine = true });
                }
                {
                    var pts = new LocalDecl("pts", new TypeRef(testStruct, null, null, 1), new AddressOf(ts));
                    unsafeBlock.Add(new Fixed(pts, new Assignment(i, new PointerMember(pts, PropertyRef.Find(testStruct, "Num")))) { IsSingleLine = true });
                }
                {
                    var p1 = new LocalDecl("p1", typeof(char*), s);
                    var p2 = new LocalDecl("p2", typeof(char*), "xy");
                    unsafeBlock.Add(new Fixed(new MultiLocalDecl(p1, p2), new Block(
                        new Assignment(new PointerIndirection(p1), new Index(p2, 1)), new Assignment(new Index(p1, 0), new PointerIndirection(p2)))));
                }
            }
            @namespace.Add(Unsafe);


            @namespace.Add(new RegionDirective("/* LINQ */") { NewLines = 2 });
            ClassDecl LinqTest = new ClassDecl("LinqTest") { NewLines = 2 };
            {
                var Person = new ClassDecl("Person");
                var Person_FirstName = new PropertyDecl("FirstName", typeof(string), Modifiers.Public) { IsAutomatic = true };
                var Person_LastName = new PropertyDecl("LastName", typeof(string), Modifiers.Public) { IsAutomatic = true };
                Person.Add(Person_FirstName, Person_LastName);
                LinqTest.Add(Person);

                var Pet = new ClassDecl("Pet");
                var Pet_Name = new PropertyDecl("Name", typeof(string), Modifiers.Public) { IsAutomatic = true };
                var Pet_Owner = new PropertyDecl("Owner", Person, Modifiers.Public) { IsAutomatic = true };
                Pet.Add(Pet_Name, Pet_Owner);
                LinqTest.Add(Pet);

                GenericMethodDecl Test;
                {
                    var TR = new TypeParameter("TR");
                    var func = new ParameterDecl("func", new TypeRef(typeof(Func<,>), Person, TR));
                    Test = new GenericMethodDecl("Test", TR, Modifiers.Static, TR);
                    Test.AddParameters(func);
                    Test.Add(new Return(new Call(func, new NewObject(Person))));
                }
                LinqTest.Add(Test);

                MethodDecl QueryExamples = new MethodDecl("QueryExamples", typeof(void), Modifiers.Public | Modifiers.Static);
                {
                    var magnus = new LocalDecl("magnus", new VarTypeRef(), new NewObject(Person) { Initializer = new Initializer(new Assignment(Person_FirstName, "Magnus"), new Assignment(Person_LastName, "Hedlund")) });
                    var terry = new LocalDecl("terry", new VarTypeRef(), new NewObject(Person) { Initializer = new Initializer(new Assignment(Person_FirstName, "Terry"), new Assignment(Person_LastName, "Adams")) });
                    var charlotte = new LocalDecl("charlotte", new VarTypeRef(), new NewObject(Person) { Initializer = new Initializer(new Assignment(Person_FirstName, "Charlotte"), new Assignment(Person_LastName, "Weiss")) });
                    var arlene = new LocalDecl("arlene", new VarTypeRef(), new NewObject(Person) { Initializer = new Initializer(new Assignment(Person_FirstName, "Arlene"), new Assignment(Person_LastName, "Huff")) });

                    var barley = new LocalDecl("barley", new VarTypeRef(), new NewObject(Pet) { Initializer = new Initializer(new Assignment(Pet_Name, "Barley"), new Assignment(Pet_Owner, terry)) }) { NewLines = 2 };
                    var boots = new LocalDecl("boots", new VarTypeRef(), new NewObject(Pet) { Initializer = new Initializer(new Assignment(Pet_Name, "Boots"), new Assignment(Pet_Owner, terry)) });
                    var whiskers = new LocalDecl("whiskers", new VarTypeRef(), new NewObject(Pet) { Initializer = new Initializer(new Assignment(Pet_Name, "Whiskers"), new Assignment(Pet_Owner, charlotte)) });
                    var bluemoon = new LocalDecl("bluemoon", new VarTypeRef(), new NewObject(Pet) { Initializer = new Initializer(new Assignment(Pet_Name, "Blue Moon"), new Assignment(Pet_Owner, terry)) });
                    var daisy = new LocalDecl("daisy", new VarTypeRef(), new NewObject(Pet) { Initializer = new Initializer(new Assignment(Pet_Name, "Daisy"), new Assignment(Pet_Owner, magnus)) });

                    var people = new LocalDecl("people", new VarTypeRef(), new NewObject(new TypeRef(typeof(List<>), Person)) { Initializer = new Initializer(magnus, terry, charlotte, arlene) }) { NewLines = 2 };
                    var pets = new LocalDecl("pets", new VarTypeRef(), new NewObject(new TypeRef(typeof(List<>), Pet)) { Initializer = new Initializer(barley, boots, whiskers, bluemoon, daisy) });

                    QueryExamples.Add(magnus, terry, charlotte, arlene, barley, boots, whiskers, bluemoon, daisy, people, pets);

                    {
                        var person = new ParameterDecl("person", new VarTypeRef());
                        QueryExamples.Add(new LocalDecl("f1", new TypeRef(typeof(Func<,>), Person, typeof(string)), new LambdaExpression(new Dot(person, Person_FirstName), person)) { NewLines = 2 });
                    }
                    {
                        var person = new ParameterDecl("person", Person);
                        QueryExamples.Add(new Call(new MethodRef(Test, typeof(string)), new LambdaExpression(new Dot(person, Person_FirstName), person)));
                    }
                    {
                        var person = new ParameterDecl("person", new VarTypeRef());
                        QueryExamples.Add(new Call(new MethodRef(Test, typeof(string)), new LambdaExpression(new Dot(person, Person_FirstName), person)));
                    }
                    {
                        var person = new ParameterDecl("person", new VarTypeRef());
                        QueryExamples.Add(new Call(new MethodRef(Test), new LambdaExpression(new Dot(person, Person_FirstName), person)));
                    }

                    LocalDecl who;
                    {
                        var person = new LocalDecl("person", new VarTypeRef());
                        who = new LocalDecl("who", new VarTypeRef(), new Call(new Dot(new QueryExpression(new SelectClause(person),
                            new FromClause(person, people)) { HasParens = true },
                            MethodRef.Find(typeof(Enumerable), "SingleOrDefault", typeof(IEnumerable<>))))) { NewLines = 2 };
                        QueryExamples.Add(who);
                    }
                    QueryExamples.Add(new LocalDecl("firstName", typeof(string), new Dot(who, Person_FirstName)));

                    LocalDecl query1;
                    {
                        var person = new LocalDecl("person", new VarTypeRef());
                        query1 = new LocalDecl("query1", new VarTypeRef(), new QueryExpression(new SelectClause(new NewObject(new AnonymousTypeRef())
                            { Initializer = new Initializer(new Dot(person, Person_FirstName)) }), new FromClause(person, people))) { NewLines = 2 };
                        QueryExamples.Add(query1);
                    }
                    LocalDecl trans1;
                    {
                        var person = new ParameterDecl("person", new VarTypeRef());
                        trans1 = new LocalDecl("trans1", new VarTypeRef(), new Call(new Dot(people, MethodRef.Find(typeof(Enumerable), "Select", typeof(IEnumerable<>), typeof(Func<,>))),
                            new LambdaExpression(new NewObject(new AnonymousTypeRef()) { Initializer = new Initializer(new Dot(person, Person_FirstName)) }, person)));
                        QueryExamples.Add(trans1);
                    }

                    LocalDecl query2;
                    {
                        var person = new LocalDecl("person", new VarTypeRef());
                        var pet = new LocalDecl("pet", new VarTypeRef());
                        query2 = new LocalDecl("query2", new VarTypeRef(), new QueryExpression(new GroupClause(pet, new Dot(pet, Pet_Owner)),
                            new FromClause(person, people), new JoinClause(pet, pets, person, new Dot(pet, Pet_Owner)))) { NewLines = 2 };
                        QueryExamples.Add(query2);
                    }
                    LocalDecl trans2;
                    {
                        var person1 = new ParameterDecl("person", new VarTypeRef());
                        var pet1 = new ParameterDecl("pet", new VarTypeRef());
                        var person2 = new ParameterDecl("person", new VarTypeRef());
                        var pet2 = new ParameterDecl("pet", new VarTypeRef());
                        ClassDecl a3 = new ClassDecl("a3", Modifiers.Private) { IsGenerated = true };
                        a3.AddProperty("person", Person);
                        var a3_pet = a3.AddProperty("pet", Pet);
                        var a3_ref = new AnonymousTypeRef();
                        a3_ref.SetReference(a3);
                        var t1 = new ParameterDecl("t", new VarTypeRef());
                        var t2 = new ParameterDecl("t", new VarTypeRef());
                        trans2 = new LocalDecl("trans2", new VarTypeRef(), new Call(new Dot(new Call(new Dot(people,
                            MethodRef.Find(typeof(Enumerable), "Join", typeof(IEnumerable<>), typeof(IEnumerable<>), typeof(Func<,>), typeof(Func<,>), typeof(Func<,,>))),
                            pets, new LambdaExpression(person1.CreateRef(), person1), new LambdaExpression(new Dot(pet1, Pet_Owner), pet1),
                            new LambdaExpression(new NewObject(a3_ref) { Initializer = new Initializer(person2, pet2) }, person2, pet2)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "GroupBy", typeof(IEnumerable<>), typeof(Func<,>), typeof(Func<,>))),
                            new LambdaExpression(new Dot(new Dot(t1, a3_pet), Pet_Owner), t1),
                            new LambdaExpression(new Dot(t2, a3_pet), t2)));
                        QueryExamples.Add(trans2);
                    }

                    LocalDecl query3;
                    {
                        var person = new LocalDecl("person", new VarTypeRef());
                        var pet = new LocalDecl("pet", new VarTypeRef());
                        var gj = new LocalDecl("gj", new VarTypeRef());
                        var subpet = new LocalDecl("subpet", Pet);
                        var p = new LocalDecl("p", new VarTypeRef(), new Index(people, 1));

                        ClassDecl a4 = new ClassDecl("a2", Modifiers.Private) { IsGenerated = true };
                        a4.AddProperty("FirstName", typeof(string));
                        var a4_PetName = a4.AddProperty("PetName", typeof(string));
                        var a4_ref = new AnonymousTypeRef();
                        a4_ref.SetReference(a4);

                        var g2 = new LocalDecl("g2", new VarTypeRef());
                        query3 = new LocalDecl("query3", new VarTypeRef(), new QueryExpression(
                            new SelectClause(new NewObject(a4_ref) { Initializer = new Initializer(new Dot(person, Person_FirstName),
                                new Assignment(a4_PetName, new Conditional(new Equal(subpet, new Literal(null)) { HasParens = false }, "", new Dot(subpet, Pet_Name)))) }),
                            new FromClause(person, people), new JoinClause(pet, pets, person, new Dot(pet, Pet_Owner), gj),
                            new FromClause(subpet, new Call(new Dot(gj, MethodRef.Find(typeof(Enumerable), "DefaultIfEmpty", typeof(IEnumerable<>))))),
                            new LetClause(p), new WhereClause(new Call(new Dot(people, MethodRef.Find(people.EvaluateType(), "Contains", typeof(TypeUtil.T))), p)),
                            new OrderByClause(new Ordering(new Dot(person, Person_LastName)), new Ordering(new Dot(person, Person_FirstName), OrderingDirection.Ascending), new Ordering(new Dot(subpet, Pet_Name), OrderingDirection.Descending))
                            ) { QueryContinuation = new QueryContinuation(new GroupClause(g2, new Dot(g2, Person_FirstName)), g2) }) { NewLines = 2 };
                        QueryExamples.Add(query3);
                    }
                    LocalDecl trans3;
                    {
                        var person1 = new ParameterDecl("person", new VarTypeRef());
                        var pet1 = new ParameterDecl("pet", new VarTypeRef());
                        var person2 = new ParameterDecl("person", new VarTypeRef());
                        var gj = new ParameterDecl("gj", new VarTypeRef());
                        var t1 = new ParameterDecl("t1", new VarTypeRef());
                        var t2 = new ParameterDecl("t2", new VarTypeRef());
                        var subpet = new ParameterDecl("subpet", new VarTypeRef());
                        var t3 = new ParameterDecl("t3", new VarTypeRef());
                        var t4 = new ParameterDecl("t4", new VarTypeRef());
                        var t5 = new ParameterDecl("t5", new VarTypeRef());
                        var t6 = new ParameterDecl("t6", new VarTypeRef());
                        var t7 = new ParameterDecl("t7", new VarTypeRef());
                        var t8 = new ParameterDecl("t8", new VarTypeRef());
                        var g2 = new ParameterDecl("g2", new VarTypeRef());

                        ClassDecl a5 = new ClassDecl("a5", Modifiers.Private) { IsGenerated = true };
                        var a5_person = a5.AddProperty("person", Person);
                        var a5_gj = a5.AddProperty("gj", new TypeRef(typeof(IEnumerable), Pet));
                        var a5_ref = new AnonymousTypeRef();
                        a5_ref.SetReference(a5);

                        ClassDecl a6 = new ClassDecl("a6", Modifiers.Private) { IsGenerated = true };
                        a6.AddProperty("t2", a5);
                        var a6_subpet = a6.AddProperty("subpet", new TypeRef(typeof(IEnumerable), Pet));
                        var a6_ref = new AnonymousTypeRef();
                        a6_ref.SetReference(a6);

                        ClassDecl a7 = new ClassDecl("a7", Modifiers.Private) { IsGenerated = true };
                        a7.AddProperty("t3", a6);
                        var a7_p = a7.AddProperty("p", Person);
                        var a7_ref = new AnonymousTypeRef();
                        a7_ref.SetReference(a7);

                        ClassDecl a8 = new ClassDecl("a8", Modifiers.Private) { IsGenerated = true };
                        a8.AddProperty("FirstName", typeof(string));
                        var a8_PetName = a8.AddProperty("PetName", typeof(string));
                        var a8_ref = new AnonymousTypeRef();
                        a8_ref.SetReference(a8);

                        trans3 = new LocalDecl("trans3", new VarTypeRef(), new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(new Call(new Dot(
                            people, MethodRef.Find(typeof(Enumerable), "GroupJoin", typeof(IEnumerable<>), typeof(IEnumerable<>), typeof(Func<,>), typeof(Func<,>), typeof(Func<TypeUtil.T, IEnumerable<TypeUtil.T>, TypeUtil.T>))),
                            pets, new LambdaExpression(person1.CreateRef(), person1), new LambdaExpression(new Dot(pet1, Pet_Owner), pet1),
                            new LambdaExpression(new NewObject(a5_ref) { Initializer = new Initializer(person2, gj) }, person2, gj)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "SelectMany", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, IEnumerable<TypeUtil.T>>), typeof(Func<,,>))),
                            new LambdaExpression(new Call(new Dot(new Call(new Dot(new Dot(t1, a5_gj), MethodRef.Find(typeof(Enumerable), "DefaultIfEmpty", typeof(IEnumerable<>)))),
                                MethodRef.Find(typeof(Enumerable), "Cast", typeof(IEnumerable)).MakeGenericRef(Pet))), t1), new LambdaExpression(new NewObject(a6_ref) { Initializer = new Initializer(t2, subpet) }, t2, subpet)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "Select", typeof(IEnumerable<>), typeof(Func<,>))), new LambdaExpression(new NewObject(a7_ref) { Initializer = new Initializer(t3, new Assignment(a7_p, new Index(people, 1))) }, t3)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "Where", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, bool>))), new LambdaExpression(new Call(new Dot(people, MethodRef.Find(people.EvaluateType(), "Contains", typeof(TypeUtil.T))), new Dot(t4, a7_p)), t4)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "OrderBy", typeof(IEnumerable<>), typeof(Func<,>))), new LambdaExpression(new Dot(new Dot(new Dot(new Dot(t5, t3), t2), a5_person), Person_LastName), t5)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Queryable), "ThenBy", typeof(IOrderedQueryable<>), typeof(Expression<Func<TypeUtil.T, TypeUtil.T>>))), new LambdaExpression(new Dot(new Dot(new Dot(new Dot(t6, t3), t2), a5_person), Person_FirstName), t6)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Queryable), "ThenByDescending", typeof(IOrderedQueryable<>), typeof(Expression<Func<TypeUtil.T, TypeUtil.T>>))), new LambdaExpression(new Dot(new Dot(new Dot(t7, t3), a6_subpet), Pet_Name), t7)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "Select", typeof(IEnumerable<>), typeof(Func<,>))), new LambdaExpression(new NewObject(new AnonymousTypeRef())
                                { Initializer = new Initializer(new Dot(new Dot(new Dot(new Dot(t8, t3), t2), a5_person), Person_FirstName),
                                    new Assignment(a8_PetName, new Conditional(new Equal(new Dot(new Dot(t8, t3), a6_subpet), new Literal(null)) { HasParens = false }, "", new Dot(new Dot(new Dot(t8, t3), a6_subpet), Pet_Name)))) }, t8)) { IsFirstOnLine = true },
                            MethodRef.Find(typeof(Enumerable), "GroupBy", typeof(IEnumerable<>), typeof(Func<,>), typeof(Func<,>))), new LambdaExpression(new Dot(g2, Person_FirstName), g2))
                        );
                        QueryExamples.Add(trans3);
                    }

                    LocalDecl b;
                    {

                        var person = new ParameterDecl("person", new VarTypeRef());
                        b = new LocalDecl("b", typeof(bool), new Call(new Dot(people, MethodRef.Find(typeof(Enumerable), "All", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, bool>))),
                            new LambdaExpression(new Equal(person, magnus), person))) { NewLines = 2 };
                        QueryExamples.Add(b);
                    }
                    LocalDecl block = new LocalDecl("block", typeof(Block), new NewObject(typeof(Block))); QueryExamples.Add(block);
                    {
                        var x = new ParameterDecl("x", new VarTypeRef());
                        QueryExamples.Add(new Assignment(b, new Call(new Dot(block, MethodRef.Find(typeof(Enumerable), "All", typeof(IEnumerable<>), typeof(Func<TypeUtil.T, bool>))), new LambdaExpression(new Equal(x, block), x))));
                    }
                }
                LinqTest.Add(QueryExamples);
            }

            @namespace.Add(LinqTest);
            @namespace.Add(new EndRegionDirective { NewLines = 2 });


            @namespace.Add(new RegionDirective("/* Inline Comments */") { NewLines = 2 });
            ClassDecl CommentTest = new ClassDecl("CommentTest") { NewLines = 2 };
            {
                MethodDecl Test = new MethodDecl("Test", typeof(void));
                {
                    var a = new LocalDecl("a", typeof(int), 1);
                    var b = new LocalDecl("b", typeof(int), 2);
                    b.AttachAnnotation(new Comment("bb") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                    b.AttachAnnotation(new Comment("b=2", CommentFlags.EOL), AnnotationFlags.NoSpace);
                    var l3 = new Literal(3);
                    l3.AttachAnnotation(new Comment("b3") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                    var c = new LocalDecl("c", typeof(int), l3);
                    var z = new LocalDecl("z", typeof(int));
                    z.AttachAnnotation(new Comment("bz") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                    var multi = new MultiLocalDecl(a, b, c, z) { EOLComment = "EOL" };
                    multi.AttachAnnotation(new Comment("before") { IsFirstOnLine = false });
                    Test.Add(multi);

                    {
                        var a_ref = a.CreateRef();
                        a_ref.AttachAnnotation(new Comment("ba") { IsFirstOnLine = false });
                        a_ref.AttachAnnotation(new Comment("aa", CommentFlags.EOL));
                        var b_ref = b.CreateRef();
                        b_ref.AttachAnnotation(new Comment("bb") { IsFirstOnLine = false });
                        b_ref.AttachAnnotation(new Comment("ab", CommentFlags.EOL));
                        var multiply = new Multiply(a_ref, b_ref) { HasParens = true, IsFirstOnLine = true, EOLComment = "after a*b" };
                        multiply.AttachAnnotation(new Comment("before a*b #1") { IsFirstOnLine = false });
                        multiply.AttachAnnotation(new Comment("before a*b #2") { IsFirstOnLine = false });
                        var c_ref = c.CreateRef();
                        c_ref.AttachAnnotation(new Comment("bc") { IsFirstOnLine = false });
                        c_ref.AttachAnnotation(new Comment("ac", CommentFlags.EOL));
                        var add = new Add(multiply, c_ref) { HasParens = true, EOLComment = "after all" };
                        add.AttachAnnotation(new Comment("before all") { IsFirstOnLine = false });
                        var assign1 = new Assignment(z, add) { EOLComment = "EOL" };
                        Test.Add(assign1);
                    }
                    {
                        var a_ref = a.CreateRef();
                        a_ref.AttachAnnotation(new Comment("ba") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        a_ref.AttachAnnotation(new Comment("aa", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var z_ref = new Literal(0);
                        z_ref.AttachAnnotation(new Comment("b0") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        z_ref.AttachAnnotation(new Comment("a0", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var b_ref = b.CreateRef();
                        b_ref.AttachAnnotation(new Comment("bb") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        b_ref.AttachAnnotation(new Comment("ab", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var c_ref = c.CreateRef();
                        c_ref.AttachAnnotation(new Comment("bc") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        c_ref.AttachAnnotation(new Comment("ac", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var agt0 = new GreaterThan(a_ref, z_ref) { HasParens = false };
                        agt0.AttachAnnotation(new Comment("ae", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var assign2 = new Assignment(z, new Conditional(agt0, b_ref, c_ref) { HasParens = false });
                        Test.Add(assign2);
                    }
                    {
                        var a_ref = a.CreateRef();
                        a_ref.AttachAnnotation(new Comment("ba") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        a_ref.AttachAnnotation(new Comment("aa", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var z_ref = new Literal(0);
                        z_ref.AttachAnnotation(new Comment("b0") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        z_ref.AttachAnnotation(new Comment("a0", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var b_ref = b.CreateRef();
                        b_ref.AttachAnnotation(new Comment("bb") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        b_ref.AttachAnnotation(new Comment("ab", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var c_ref = c.CreateRef();
                        c_ref.AttachAnnotation(new Comment("bc") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        c_ref.AttachAnnotation(new Comment("ac", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var agt0 = new GreaterThan(a_ref, z_ref);
                        agt0.AttachAnnotation(new Comment("be") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        agt0.AttachAnnotation(new Comment("ae", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var conditional = new Conditional(agt0, b_ref, c_ref);
                        conditional.AttachAnnotation(new Comment("before") { IsFirstOnLine = false }, AnnotationFlags.NoSpace);
                        conditional.AttachAnnotation(new Comment("after", CommentFlags.EOL), AnnotationFlags.NoSpace);
                        var assign3 = new Assignment(z, conditional);
                        assign3.AttachAnnotation(new Comment("bz") { IsFirstOnLine = false });
                        Test.Add(assign3);
                    }
                    {
                        var ifAssign = new Assignment(b, 1);
                        ifAssign.AttachAnnotation(new Comment("bb"), AnnotationFlags.NoSpace);
                        var agt0 = new GreaterThan(a, 0);
                        agt0.AttachAnnotation(new Comment("ba"), AnnotationFlags.NoSpace);
                        var @if = new If(agt0, ifAssign);
                        @if.AttachAnnotation(new Comment("bif") { IsFirstOnLine = false });
                        ifAssign.AttachAnnotation(new Comment("EOL", CommentFlags.EOL | CommentFlags.Block));
                        @if.IsSingleLine = true;
                        Test.Add(@if);
                    }
                }
                CommentTest.Add(Test);
            }
            @namespace.Add(CommentTest);
            @namespace.Add(new EndRegionDirective { NewLines = 2 });


            @namespace.Add(new RegionDirective("/* External Library Tests */") { NewLines = 2 });
            ClassDecl ExternalTests = new ClassDecl("ExternalTests") { NewLines = 2 };
            {
                var TestLibrary = project.FindRef("TestLibrary");
                MethodDecl Test = new MethodDecl("Test", typeof(void), Modifiers.Unsafe);
                {
                    var Test1 = (TypeRefBase)TypeRef.Find(TestLibrary, "Test1");
                    var test = new LocalDecl("test", new Dot(TestLibrary, Test1), new NewObject(new Dot(TestLibrary, Test1))); Test.Add(test);
                    var t1 = new LocalDecl("t1", new VarTypeRef(), new Dot(test, FieldRef.Find(Test1, "_enum1"))); Test.Add(t1);
                    var t2 = new LocalDecl("t2", new VarTypeRef(), new Dot(test, FieldRef.Find(Test1, "_volatile"))); Test.Add(t2);
                    var t3 = new LocalDecl("t3", typeof(int*), new Dot(test, FieldRef.Find(Test1, "_unsafePointer"))); Test.Add(t3);
                    var e = new LocalDecl("e", new Dot(TestLibrary, TypeRef.Find(TestLibrary, "Enum1"))); Test.Add(e);
                    var s = new LocalDecl("s", new Dot(TestLibrary, TypeRef.Find(TestLibrary, "Struct1"))); Test.Add(s);

                    var Test1_InnerTest1 = (TypeRefBase)TypeRef.Find(Test1, "InnerTest1`1");
                    var it = new LocalDecl("it", new Dot(new Dot(TestLibrary, Test1), Test1_InnerTest1.MakeGenericRef(typeof(int))),
                        new NewObject(new Dot(new Dot(TestLibrary, Test1), Test1_InnerTest1.MakeGenericRef(typeof(int))))) { NewLines = 2 }; Test.Add(it);
                    var GenericMethod = MethodRef.Find(Test1_InnerTest1, "GenericMethod", typeof(TypeUtil.T), typeof(TypeUtil.T), typeof(TypeUtil.T));
                    Test.Add(new Assignment(test, new Call(new Dot(it, GenericMethod.MakeGenericRef(TestClass, new Dot(TestLibrary, Test1), new Dot(TestLibrary, Test1))), 1, new NewObject(TestClass), test)));

                    var UnsafeClass = (TypeRefBase)TypeRef.Find(TestLibrary, "UnsafeClass");
                    var uc = new LocalDecl("uc", new Dot(TestLibrary, UnsafeClass), new NewObject(new Dot(TestLibrary, UnsafeClass))) { NewLines = 2 }; Test.Add(uc);
                    Test.Add(new Call(new Dot(uc, MethodRef.Find(uc.EvaluateType(), "DoUnsafeStuff"))));
                    var SafeClass = (TypeRefBase)TypeRef.Find(TestLibrary, "SafeClass");
                    var sc = new LocalDecl("sc", new Dot(TestLibrary, SafeClass), new NewObject(new Dot(TestLibrary, SafeClass))); Test.Add(sc);
                    Test.Add(new Call(new Dot(sc, MethodRef.Find(sc.EvaluateType(), "DoUnsafeStuff"))));

                    var BaseTest = (TypeRefBase)TypeRef.Find(TestLibrary, "BaseTest");
                    var b = new LocalDecl("b", new Dot(TestLibrary, BaseTest), new NewObject(new Dot(TestLibrary, BaseTest))) { NewLines = 2 }; Test.Add(b);
                    Test.Add(new Call(new Dot(b, MethodRef.Find(b.EvaluateType(), "OverriddenMethod"))));
                    Test.Add(new Call(new Dot(b, MethodRef.Find(b.EvaluateType(), "HiddenMethod"))));
                    var DerivedTest = (TypeRefBase)TypeRef.Find(TestLibrary, "DerivedTest");
                    var d = new LocalDecl("d", new Dot(TestLibrary, DerivedTest), new NewObject(new Dot(TestLibrary, DerivedTest))); Test.Add(d);
                    Test.Add(new Call(new Dot(d, MethodRef.Find(d.EvaluateType(), "OverriddenMethod"))));
                    Test.Add(new Call(new Dot(d, MethodRef.Find(DerivedTest, "HiddenMethod", BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Public, false))));

                    var BaseTest_NestedClass = (TypeRefBase)TypeRef.Find(BaseTest, "NestedClass");
                    var bnc = new LocalDecl("bnc", Dot.Create(TestLibrary, BaseTest, BaseTest_NestedClass), new NewObject(Dot.Create(TestLibrary, BaseTest, BaseTest_NestedClass))); Test.Add(bnc);
                    var DerivedTest_NestedClass = (TypeRefBase)TypeRef.Find(DerivedTest, "NestedClass");
                    var dnc = new LocalDecl("dnc", Dot.Create(TestLibrary, DerivedTest, DerivedTest_NestedClass), new NewObject(Dot.Create(TestLibrary, DerivedTest, DerivedTest_NestedClass))); Test.Add(dnc);
                }
                ExternalTests.Add(Test);
            }

            @namespace.Add(ExternalTests);
            @namespace.Add(new EndRegionDirective { NewLines = 2 });


            return codeUnit;
        }
    }
}
