﻿// Nova.CodeDOM usage examples.
// Copyright (C) 2007-2011 Inevitable Software, all rights reserved.

using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;

using Nova.CodeDOM;
using Nova.Utilities;

namespace Nova.Test
{
    /// <summary>
    /// This program contains tests for the Nova.CodeDOM C# object model library.
    /// </summary>
    class Program
    {
        #region /* MAIN METHOD */

        static void Main()
        {
            // Determine the application name and if we're running in VS
            string appName = AppDomain.CurrentDomain.FriendlyName;
            if (appName.EndsWith(".exe"))
                appName = appName.Substring(0, appName.Length - 4);
            if (appName.EndsWith(".vshost"))
                appName = appName.Substring(0, appName.Length - 7);

            Version appVersion = typeof(Program).Assembly.GetName().Version;
            string appDescription = appName + " " + appVersion.Major + "." + appVersion.Minor;
            Console.WriteLine(appDescription + "  -  C# CodeDOM Test Utility.\nCopyright (C) 2011 Inevitable Software, all rights reserved.");

            // Execute all public tests in this class
            Log.WriteLine("\nExecuting Nova.CodeDOM tests...\n");
            MethodInfo[] methodInfos = Assembly.GetExecutingAssembly().GetType("Nova.Test.Program").GetMethods(BindingFlags.Public | BindingFlags.Static);
            foreach (var methodInfo in methodInfos)
            {
                Log.WriteLine("===== Executing Test: " + methodInfo.Name + " =====");
                try
                {
                    methodInfo.Invoke(null, null);
                }
                catch (Exception ex)
                {
                    Log.WriteLine("EXCEPTION: " + ex.Message);
                }
                Log.WriteLine("===== " + methodInfo.Name + " Completed, Press Enter to Continue =====\n");
            }

            // Wait for a key press before exiting if running in VS
            if (Debugger.IsAttached)
            {
                Console.WriteLine("DONE.  Press any key to exit.");
                Console.ReadKey();
            }
        }

        #endregion

        #region /* TESTS */

        /// <summary>
        /// Load the FullTest.cs file, and save it as FullTest.saved.cs.
        /// </summary>
        public static void SaveFullTest()
        {
            string filePath = FileUtil.FindFile("FullTest.cs");
            if (filePath != null)
            {
                // Use a separate project for this test, with special references
                string baseDirectory = FileUtil.GetBaseDirectory();
                var project = CreateMiscellaneousProject("FullTest", new AssemblyReference("Nova.CodeDOM", null, null, false, baseDirectory),
                    new AssemblyReference("Nova.Test.Library", null, null, false, baseDirectory), new AssemblyReference("EnvDTE80", "EnvV1"), new AssemblyReference("EnvDTE90", "EnvV2"));
                CodeUnit codeUnit = project.AddFile(filePath);
                project.ParseAndResolveCodeUnits();
                string fileName = Path.ChangeExtension(filePath, ".saved.cs");
                Log.WriteLine("Saving '" + fileName + "'...");
                codeUnit.SaveAs(fileName);
            }
        }

        /// <summary>
        /// Manually generate the code in FullTest.cs, save it in FullTest.generated.cs, and compare to FullTest.saved.cs.
        /// </summary>
        public static void GenerateFullTest()
        {
            string baseDirectory = FileUtil.GetBaseDirectory();
            var project = CreateMiscellaneousProject("FullTest.generated",
                new AssemblyReference("Nova.CodeDOM", null, null, false, baseDirectory),
                new AssemblyReference("Nova.Test.Library", null, null, false, baseDirectory),
                new AssemblyReference("EnvDTE80", "EnvV1"), new AssemblyReference("EnvDTE90", "EnvV2"));
            const string fileName = "FullTest.generated.cs";
            Log.WriteLine("Generating code for '" + fileName + "'...");
            var codeUnit = ManualTests.GenerateFullTest(fileName, project);
            Log.WriteLine("Saving '" + fileName + "'...");
            codeUnit.Save();

            string file2 = FileUtil.FindFile("FullTest.saved.cs");
            CompareFiles(fileName, file2);
        }

        private static Project CreateMiscellaneousProject(string name, params ReferenceBase[] references)
        {
            // Create a miscellaneous project file, and add typical references
            var project = new Project(name, null);
            project.AddDefaultAssemblyReferences();
            project.AddAssemblyReference("System.Configuration");
            project.AddAssemblyReference("PresentationCore");
            project.AddAssemblyReference("PresentationFramework");
            project.AddAssemblyReference("UIAutomationProvider");
            project.AddAssemblyReference("WindowsBase");
            foreach (var reference in references)
                project.References.Add(reference);
            project.LoadReferencedAssembliesAndTypes(true);
            return project;
        }

        private static void CompareFiles(string fileName1, string fileName2)
        {
            Log.WriteLine("Comparing '" + fileName1 + "' to '" + fileName2 + "'...");
            int line = 0;
            using (var fileStream1 = new StreamReader(fileName1))
            using (var fileStream2 = new StreamReader(fileName2))
            {
                while (!fileStream1.EndOfStream && !fileStream2.EndOfStream)
                {
                    string file1Line = fileStream1.ReadLine();
                    string file2Line = fileStream2.ReadLine();
                    ++line;
                    if (file1Line != file2Line)
                    {
                        Log.WriteLine("FAIL: Files differ at line " + line + "!");
                        return;
                    }
                }
                if (!fileStream1.EndOfStream)
                    Log.WriteLine("FAIL: First file is longer!");
                else if (!fileStream1.EndOfStream)
                    Log.WriteLine("FAIL: Second file is longer!");
                else
                    Log.WriteLine("PASS: Files are identical.");
            }
        }

        #endregion
    }
}
